import time
import sys
import os
from pynput.keyboard import Controller
from os import path
global_command = [0]
def WriteCommand(s,terminal=False):
    pdir=path.abspath(' ')[:-2]
    global_command[0] = s
def LaTeX(s):
    print('LaTeX')
    os.system('/usr/local/texlive/2015/bin/x86_64-linux/xelatex '+s)
    com='/usr/bin/evince '+path.splitext(s)[0]+'.pdf '
    WriteCommand(com)
def Fortran(s):
    print('Fortran')
    Cpp(s,g='gfortran')
def C(s):
    print('C')
    Cpp(s,g='g++')
def Asm(s):
    filename=s[0]
    t=path.splitext(filename)[0]
    os.system('nasm -f elf64 %s -o %s.o' % (filename,t))
    os.system('ld -s -o %s.out %s.o' % (t,t))
    WriteCommand('./%s.out' % (path.split(t)[1],),terminal=True)
def Cpp(comt,g='g++'):
    if g=='g++':print('C++')
    s=comt[0]
    if g in ['gcc','g++']:bs=' -o3 '
    else:bs=' '
    com=g+bs+s+' -o '+path.splitext(s)[0]+'.out'
    os.system(com)
    filex=path.split(path.splitext(s)[0]+'.out')[1]
    com='./'+filex
    print(com)
    WriteCommand(com,terminal=True)
def Python(c1,c2):
    print(c2)
    pdir=path.abspath(' ')
    com=c2+' '+c1
    WriteCommand(com,terminal=True)
def Make(f):
    print('make')
    WriteCommand('make',True)
def main(com):
    default='python3'
    if com[1]=='LaTeX':LaTeX(com[0])
    elif com[1] in ['python','python3' ,'pypy','pypy3','ipython','ipython3']:Python(com[0],com[1])
    elif com[1]=='Fortran':Fortran(com)
    elif com[1]=='cpp':Cpp(com)
    elif com[1]=='c':C(com)
    else:
        x=path.splitext(com[0])[1]
        if x in ['','.py']:
            s=open(com[0],'r').readline().strip()
            s=s.split()
            if len(s)>=2:s=s[-1]
            else:s=default
            if s.lower().find('py')!=-1:Python(com[0],s.lower())
            elif x=='' and path.split(com[0])[-1].lower()=='makefile':Make(com[0])
            else:Python(com[0],default)
        if x.lower() in ['.tex']:LaTeX(com[0])
        if x.lower() in ['.gf','.f','.f90']:Fortran(com)
        if x.lower() in ['.c']:C(com)
        if x.lower() in ['.cpp']:Cpp(com)
        if x.lower() in ['.asm']:Asm(com)

def run_ter(s):
    os.system(sys.argv[1])
    time.sleep(float(sys.argv[2]))
    keyboard = Controller()
    keyboard.type(s)
    keyboard.type('\n')
if __name__=='__main__':
    main([sys.argv[3],''])
    run_ter(global_command[0])


import fire
import requests
import re,os
from multiprocessing.pool import ThreadPool
from os import path
from hashlib import md5
from bs4 import BeautifulSoup 
import sys
import shelve
import time
import copy
__version__='0.1.3'
__author__="Blacksong"
class Wget:
    def __init__(self,url,**kargs):
        url0=url
        url=url.split('?')[0]
        self.n=0
        dbname=md5(url.encode()).hexdigest()[:7]+'.pydb'

        self.filetype=kargs.get('filetype','.jpg')
        self.iterarion = kargs.get('iter','NO')
        if self.iterarion == 'NO':
            record_db = dict()
        else:
            record_db=shelve.open(dbname)

        self.headers=self.setheaders()
        self.srcpro=None
        self.rule=kargs.get('rule',list())
        self.re_rule=kargs.get('re_rule',list())
        self.max_download=8
        self.num_download = 0
        self.asyncThread=ThreadPool(self.max_download)
        self.htm=url.split('/')[2]
        print(self.htm,'htm')
        dirname = kargs.get('dirname',None)
        if dirname is None:
            dirname = self.htm
        if not path.isdir(dirname): 
            os.makedirs(dirname)
        self.dirname = dirname
        self.auto=kargs.get('auto',True)
        print(self.htm)

        self.rule_list=[re.sub('[^A-Za-z]','', url)]
        [self.rule_list.append(re.sub('[^A-Za-z]','', i)) for i in self.rule]
        self.rule_list=list(set(self.rule_list))
        self.rule_dir=[path.dirname(url)]
        [self.rule_dir.append(path.dirname(i)) for i in self.rule]
        self.rule_dir=list(set(self.rule_dir))

        self.re_rule=[re.compile(i) for i in self.re_rule]
        url=url0
        print(self.re_rule,'\n',self.rule_dir,'\n',self.rule_list)
        
        try:
            halt=record_db.get('halt',False)
            if halt == True:
                self.href=record_db.get('href',[(url,{})])
                self.pagedb = record_db.get('pagedb',set())
                self.srcdb = record_db.get('srcdb',set())
                record_db['halt']=False
            else:
                self.href=[(url,{})]
                self.pagedb=set()
                self.srcdb=set()
            self.autofind(url)
            self.main()
            record_db.close()
            if path.isfile(dbname): os.remove(dbname)
        except:
            print('the program is halted!')
            record_db['halt']=True
            record_db['srcdb']=self.srcdb
            record_db['pagedb']=self.pagedb
            record_db['href']=[i for i in self.href if i!=None]
        self.asyncThread.close()
        self.asyncThread.join()
    def my_hash(self,x):
        return int(md5(x.encode()).hexdigest()[:8],16)
    def setheaders(self):
        headers='''
User-Agent: Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:48.0) Gecko/20100101 Firefox/48.0
DNT: 1
Connection: keep-alive
Upgrade-Insecure-Requests: 1
Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2
Host: i.meizitu.net
Referer: http://www.mzitu.com/131504/5
'''
        headers=headers.split('\n')
        d=dict()
        for i in headers:
            n=i.find(':')
            if n ==-1:continue
            d[i[:n].strip()]=i[n+1:].strip()
        return d
    def getHost(self,url):
        return url.split('/')[2]
    def autofind(self,url):
        print('Autofind the label of the target picture')
        volume=0
        x,y=self.Analyze(url)
        
        r=set()
        for i in y:
            s=str(i[1])
            if self.iterarion == 'NO':
                self.Download(i[0],i[0])
            if s in r:continue
            r.add(s)
            try:
                headers = self.headers
                headers['Host']=self.getHost(i[0])
                headers['Referer']=url
                f=requests.get(i[0],timeout=30,headers = headers).content
                t=len(f)
                
            except:
                t=0
            print(t,i[0])
            if volume<t:
                volume=t
                tt=i[1]
        if self.iterarion == 'NO':
            sys.exit(0)

        self.srcpro=tuple(tt.items())
        print(self.srcpro)
    def main(self):
        '''主循环函数，控制寻找页面以及查找页面资源链接，主要起控制作用'''
        def run(u):
            print('Analyse the html ',u)
            hr,src=self.Analyze(u)

            for i in src:
                if self.UsefulSrc0(*i):
                    self.num_download+=1
                    self.asyncThread.apply_async(self.Download,(i[0],u))
                    while self.num_download>self.max_download+4:
                        time.sleep(1)
                    # self.Download(i[0],u)
            self.pagedb.add(self.my_hash(u))
            for i in hr:
                if self.UsefulHtml0(*i):
                    self.href.append(i)
        while True:
            ii=0
            n=len(self.href)
            while ii<n:
                if self.UsefulHtml0(*self.href[ii]):
                    run(self.href[ii][0])
                self.href[ii]=None
                ii+=1
            self.href=[i for i in self.href if i!=None]

            if len(self.href)==0 or self.iterarion=='NO':break
    def DivSplit(self,s):
        '''将一个html页面分成多个div块，主要通过寻找div标签的位置，返回一个记录了div块所在位置以及各个块的名字的链表'''
        a=[]
        [a.append((-1,i.span())) for i in re.finditer('< *div[^><]*>', s)]
        b=[]
        for i,j in a:
            if i==1:
                b.append((i,j[0]))
            else:
                t=s[j[0]:j[1]]
                n=re.findall('id *= *"[^"]*"|class *= *"[^"]*"', t)
                d=dict([i.replace('"','').split('=') for i in n])
                b.append((i,j[0],d))
        b.sort(key=lambda x:x[1])
        return b
    def DivSplit2(self,s):
        '''将一个html页面分成多个div块，主要通过寻找div标签的位置，返回一个记录了div块所在位置以及各个块的名字的链表'''
        a=[(-1,0,{'id':'mystart'})]
        for i in re.finditer('(id|class) *=["\' ]*[^"\']*', s):
            j=re.sub('["\' ]', '',i.group())
            n=j.find('=')
            d={j[:n]:j[n+1:]}
            a.append((-1,i.span()[1],d))
        a.sort(key=lambda x:x[1])
        return a
    def Download(self,url,purl,nn=[0]):
        '''下载url'''
        if self.iterarion == 'NO':
            print('Downloading ',url)
        tf=re.sub('\W','', purl)
        filename=self.dirname+'/'+tf[-min(len(tf),10):]+md5(url.encode()).hexdigest()[:5]+self.filetype
        if not path.isfile(filename):
            nn[0]+=1
            if nn[0]%50==0:
                print('Downloading ',url)
            headers = self.headers
            headers['Host']=self.getHost(url)
            headers['Referer']=purl
            x=requests.get(url,timeout=30,headers = headers)
            t=open(filename,'wb').write(x.content)
        else:
            print('file already exist')
        self.srcdb.add(self.my_hash(url))
        self.num_download -= 1
        return filename
    def UsefulHtml0(self,url,pro):
        '''判断一个页面的url是否是有用的'''
        if self.my_hash(url) in self.pagedb:return False
        if self.re_rule:
            for i in self.re_rule_list:
                if i.search(url):return True
        if not self.rule:
            if not re.search(self.htm, url):return False
        t= re.sub('[^A-Za-z]','', url)
        d=path.dirname(url)
        if t in self.rule_list:return True
        if d in self.rule_dir:return True
        if self.auto:
            return False
        return self.UsefulHtml(url,pro)
    def UsefulHtml(self,url,pro):
        '''判断一个页面的url是否是 有用的，这个函数可以在不同的环境中重写，其中pro是该链接所在div的属性'''
        return True
    def UsefulSrc0(self,url,pro):
        if self.my_hash(url) in self.srcdb:return False
        if self.auto:
            for k,v in self.srcpro:
                if v.isdigit():continue
                if pro.get(k)!=v:return False
        return self.UsefulSrc(url,pro)
    def UsefulSrc(self,url,pro):
        return True
    def correct_url(self,s,website,webdir):
        if s[0]=='/':return website+s
        elif s=='#':return ''
        elif s.find('http')!=-1:return s
        else: return webdir+s
    def Analyze(self,url):
        '''返回 href 和 src的链接，返回值为一个二元tuple'''
        headers = self.headers
        headers['Host']=self.getHost(url)
        s=requests.get(url,timeout=30,headers = headers).text
        divs=self.DivSplit(s)
        href=[]
        src=[]
        split_url=url.split('/')
        website='/'.join(split_url[:3])
        webdir='/'.join(split_url[:-1])+'/'
        for i in re.finditer(' *(href|src) *=["\' ]*[^ )("\';\+>}]+', s):
            div=self.FindDiv(divs, i.span()[0])
            j=i.group()
            j=re.sub('["\' \\\\]', '', j) #针对某些网站将url写在javascript中，用到了转义符\
            if j[0]=='h':
                j=j.replace('href=', '')
                j=self.correct_url(j,website,webdir)
                if len(j)==0:continue
                href.append((j,div))
            if j[0]=='s':
                j=j.replace('src=', '')
                if j.find(self.filetype)==-1:continue
                div=self.FindDiv(divs, i.span()[0])
                j=self.correct_url(j,website,webdir)
                if len(j)==0:continue
                src.append((j,div))
        return href,src
    def FindDiv(self,divs,pos):
        a,b=0,len(divs)
        if b==0:
            return {'id':'nodivs'}
        if pos>divs[-1][1]:return divs[-1][2]
        while b-a>1:
            t=int((a+b)/2)
            p0=divs[t][1]
            if pos>p0:a=t
            else:b=t
        return divs[a][2]
class Wget_novel:
    website = ['https://www.x83zw.com/', 'https://www.zhuaji.org/','https://www.ybdu.com/',
    'https://www.23us.so']
    trans=str.maketrans('一二三四五六七八九','123456789')
    trans2=str.maketrans(dict(zip(['零','十','百','千','万','亿'],['','0 ','00 ','000 ',' 10000 ',' 100000000 '])))
    trans.update(trans2)
    def __init__(self,url,novel_name=None,author = None,chapter_attrs=None,content_attrs=None,next_attrs=None):
        content = requests.get(url)
        bs=BeautifulSoup(content.content)

        self.re_br = re.compile('<[^>]+>')
        self.re_n = re.compile('[\n]+')
        self.re_chapter = re.compile('(第[\d]+[章节回]|第[一二三四五六七八九]+[章节回])')
        self.re_chinese_number = re.compile('[一二三四五六七八九十零百千万\d]+')

        self.author = author
        self.chapter_attrs = chapter_attrs
        self.content_attrs = content_attrs 
        self.next_attrs = next_attrs
        
        title,author = self.__get_author(bs)
        self.__auto_select(bs)
        print(self.chapter_attrs,self.content_attrs,self.next_attrs)
        print(title,author)
        if novel_name is None:fname=title+'.txt'
        else:fname=novel_name+'.txt'
        self.fp=open(fname,'w',encoding = 'utf8')
        print(fname)
        self.fp.write(title+'\n'+author+'\n')
        self.chapter=[]
        try:
            self.__start(url)
        except Exception as e:
            print(e)
        self.fp.close()
        x=set(self.chapter)
        y=set(range(1,max(self.chapter)+1))
        print(y-x)
    def div_generate(self,bs,div_list=None):
        if div_list is None:
            div_list = list()
        if bs.div is None:
            return div_list
        for i in bs.find_all('div'):
            div_list.append(i)
            self.div_generate(i,div_list)
        return div_list
    def __auto_select(self,bs):#自动识别 章 div标签 正文div标签 和下一章div标签
        div_list = self.div_generate(bs)
        if self.chapter_attrs is None:
            for i in div_list:
                t = i.text.lstrip()
                if len(t)>50:
                    continue
                if self.re_chapter.match(t):
                    self.chapter_attrs = i.attrs
                    break
            else:
                try:
                    t = self.__get_chapter(bs)
                    if not self.re_chapter.match( t[0]):
                        assert False
                except:
                    title = bs.title.text
                    t = self.re_chapter.search( title)
                    chapter = t.group()
                    t = title.split(chapter)
                    self.chapter_attrs = len(t)

        if self.content_attrs is None:
            length = 0
            for i in div_list:
                div = copy.deepcopy(i)
                for j in div.find_all('div'):
                    j.decompose()
                l = len(re.sub('\s','',div.text))
                if length<l:
                    length = l
                    self.content_attrs = div.attrs
        if self.next_attrs is None:
            for i in div_list:
                if i.div:continue
                t = re.sub('\s', '', i.text)
                if len(t)>50:
                    continue
                if t.find('下一') !=-1 and t.find('上一') != -1 and i.a is not None:
                    self.next_attrs = i.attrs
                    break

    def __start(self,url):#主循环函数
        self.url = url
        re_sub_title = re.compile('^[^\n]+\n')
        while True:
            print(url,'url')
            content = requests.get(url)
            bs = BeautifulSoup(content.content)
            title_info=self.__get_chapter(bs)
            self.chapter.append(title_info[-1])
            print(title_info)
            #必须先找下一页才能找内容，查找内容会破坏bs数据
            next_info=self.__get_next(bs)
            content_info=self.__get_content(bs)
            
            if title_info[0] is not None:
                titles = '\n\n'+title_info[0]+' '+title_info[1]+'\n\n'
                if content_info.startswith(title_info[0]):
                    content_info = re_sub_title.sub('',content_info)
                self.fp.write(titles)
                self.fp.write(content_info)
            if next_info is False:break
            url=next_info
    def __get_author(self,bs):
        title = bs.title.text.split()
        title = ''.join(title)
        return title,'作者：无名氏'
    def __get_content(self,bs):
        x = bs.find(attrs=self.content_attrs)
        if x.div:
            for i in x.find_all('div'):
                i.decompose()
        sx = str(x)

        a = x.find_all('a')
        for i in a:
            sx = sx.replace(str(i),'')
        sx = self.re_br.sub('\n',sx)
        sx = self.re_n.sub('\n',sx)
        return sx.strip()
    def __get_next(self,bs,html=None):
        c=bs.find(attrs=self.next_attrs)
        t=c.find_all('a')
        for i in t:
            if i.text.find('下')!=-1:
                href=i['href']
        if len(href)==0:return False
        if href[0]!='/':
            url=self.url.split('/')
            url[-1]=href
            return '/'.join(url)
        else:
            url=self.url.split('/')
            url = url[:3]
            url.append(href)
            return '/'.join(url)
    def __get_chapter(self,bs,html=None):
        if isinstance(self.chapter_attrs,int):
            title = bs.title.text
            t = self.re_chapter.search( title)
            chapter = t.group()
            tt = title.split(chapter)
            n = self.__ChineseNumber_to_number(self.re_chinese_number.search(chapter).group())
            tt = chapter,tt[-1],n
            return tt
            
        else:
            title=bs.find(attrs=self.chapter_attrs).h1.text
            t=self.re_chinese_number.search(title)
            if not t:return None,None,-1
            t=t.group()
            n=self.__ChineseNumber_to_number(t)
            m=title.find(t)+len(t)
            name=title[m+1:].strip()
            if name:
                if name[0]=='章' or name[0]=='节':name=name[1:]
            else:
                name = ''
            name=re.sub(':|：|,|，','',name)
            s='第'+t+'章',name,n
        return s
    def __ChineseNumber_to_number(self,s):
        t=self.trans
        s=s.translate(t)
        l=s.rstrip().split()
        n,m=0,0
        for i in l:
            j=int(i)
            if j<10000:
                if j==0:j=10
                n+=j
            else:
                n*=j
                m+=n
                n=0
        m+=n
        return m
def main(url,**karg):
    print(url,karg)
    Wget(url,**karg)
if __name__=='__main__':
    fire.Fire(main)
    # random_name('./www.35uo.com')
    # rule=['http://www.35uo.com/p05/list_37.html','http://www.35uo.com/p05/index.html']
    # Wget('https://m.733.so/mh/26798/443604.html?page=3')
    # Wget('http://www.uutu.me/index.php?c=Article&id=2527&p=2',re_rule=['id=2527&p=.*'])
    # Wget('http://m.7kk.com/picture/2918797.html')
    # Wget('http://www.umei.cc/meinvtupian/nayimeinv/27322.htm')
    # url='https://image.baidu.com/search/detail?ct=503316480&z=0&ipn=false&word=angelababy&hs=0&pn=-1&spn=0&di=baikeimg&pi=&rn=1&tn=baiduimagedetail&is=&istype=&ie=utf-8&oe=utf-8&in=&cl=2&lm=-1&st=&lpn=0&ln=undefined&fr=ala&fmq=undefined&fm=undefined&ic=&s=&se=&sme=&tab=&width=&height=&face=&cg=star&bdtype=0&oriquery=&objurl=http%3A%2F%2Fimgsrc.baidu.com%2Fbaike%2Fpic%2Fitem%2Fa1ec08fa513d2697d2379f9f50fbb2fb4216d808.jpg&fromurl=http%3A%2F%2Fbaike.baidu.com%2Fview%2F1513794.htm&gsm='
    # x=requests.get(url)
    # t=re.finditer('(href|src) *=["\' ]*(http:|https:|/)[^ )("\';>}]+', x.text)
    # for i in t:
    #     j=i.group()
    #     print(j)
    # Wget('http://www.mm131.com/xinggan/2216.html')
    # Wget('http://www.mm131.com/chemo/2043.html')
    # Wget('http://pic.yesky.com/235/108576735.shtml')
    # Wget('http://www.ycgkja.com/siwameinv/13125.html')
    # Wget('http://www.tu11.com/meituisiwatupian/2017/7933.html')
    # Wget('http://www.4493.com/siwameitui/115401/1.htm',rule=['http://www.4493.com/siwameitui/115401/1.htm','http://www.4493.com/gaoqingmeinv/115538/1.htm'])
    # Wget('http://www.522yw.mobi/article/44140.html')
    # Wget_novel('https://www.x83zw.com/book/13/13860/6646835.html')



__author__ = 'SherlockLiao'

import torch
from torchvision import models
from torch import nn


class feature_net(nn.Module):
    def __init__(self, model):
        super(feature_net, self).__init__()

        if model == 'vgg':
            vgg = models.vgg19(pretrained=True)
            self.feature = nn.Sequential(*list(vgg.children())[:-1])
            self.feature.add_module('global average', nn.AvgPool2d(9))
        elif model == 'inceptionv3':
            inception = models.inception_v3(pretrained=True)
            self.feature = nn.Sequential(*list(inception.children())[:-1])
            self.feature._modules.pop('13')
            self.feature.add_module('global average', nn.AvgPool2d(35))
        elif model == 'resnet152':
            resnet = models.resnet152(pretrained=True)
            self.feature = nn.Sequential(*list(resnet.children())[:-1])

    def forward(self, x):
        """
        model includes vgg19, inceptionv3, resnet152
        """
        x = self.feature(x)
        x = x.view(x.size(0), -1)
        return x


class classifier(nn.Module):
    def __init__(self, dim, n_classes):
        super(classifier, self).__init__()
        self.fc = nn.Sequential(
            nn.Linear(dim, 1000),
            nn.ReLU(True),
            nn.Dropout(0.5),
            nn.Linear(1000, n_classes)
        )

    def forward(self, x):
        x = self.fc(x)
        return x
if __name__ == '__main__':
    feature_net('vgg')



import numpy as np
from os.path import getsize
import pandas as pd
import re
import pytecio
__version__='1.10'
__author__='Blacksong'
class _zone_data(dict):
    plot_package = {}
    def __init__(self,*d,**dd):
        super().__init__(*d,**dd)
        self.Elements = None
        self.tec_cellcentered=False
    def _setPlot3d_type(self):#文件格式为plot3d格式时运行这个函数  增加x,y,z,ptype这个变量 
        self.x=None
        self.y=None
        self.z=None
        self.ptype=None
        
        return self
    def rename(self,old,new):
        if old == new:return
        self[new]=self[old]
        self.pop(old)
    def DataFrame(self,center=False,columns=None):
        ''''将数据转化为DataFrame格式，
        center:是否将数据都转化为中心格式
        '''
        if columns is None:
            columns = self.keys()
        dic = {i:self[i] if not center else self.center_value(i) for i in columns}
        return pd.DataFrame(dic)

    def fit_mesh(self,zone_data,var,X='x',Y='y',Z=None):
        '''
        将zong_data中的变量var,根据Elements适应,调整var中数据的存储顺序
        X,Y,Z：表述存储网格坐标的变量名
        '''
        if Z:
            mesh = [X,Y,Z]
        else:
            mesh = [X,Y]
        assert self.Elements.shape == zone_data.Elements.shape
        m1 = {i:self.center_value(i) for i in mesh}
        m2 = {i:zone_data.center_value(i) for i in mesh}
        m2['__var'] = var
        m1 = pd.DataFrame(m1)
        m2 = pd.DataFrame(m2)
        m2.index = m1.index
        m1.sort_values(mesh, inplace=True)
        m2.sort_values(mesh, inplace=True)
        
        m1['__var'] = m2['__var'].values
        m1.sort_index(inplace=True)
        return m1['__var'].values
    def set_data(self,names,values,attribute):
        self.names=names
        self.data=values
        self.attribute=attribute
        for i,v in zip(names,values):
            self[i]=v
    def __getitem__(self,k):
        if isinstance(k,str):
            return super().__getitem__(k)
        else:
            if self.attribute == 'fluent_prof':
                return self.data[:,k]
    def is_centered(self,name): #判断一个tecplot变量是不是centered变量
        nodes = int(self.attribute['Nodes'])
        if len(self[name])==nodes:
            return False
        else:
            return True

    def center_value(self,name):# 获取tecplot文件中某个变量在cell中心的值，即求出各个节点的平均值
        elements = self.Elements - 1
        n = elements.shape[1]
        elements_flat = elements.flatten()
        data = self[name][elements_flat].reshape((-1,n))
        return data.mean(1)
    def __add__(self,other):#重载加法运算
        z = class_read()
        names = list(self.keys())
        values = [self[i] + other[i] for i in names]
        z.update(zip(names,values))
        return z
    def __sub__(self,other):#重载减法运算
        z = class_read()
        names = list(self.keys())
        values = [self[i] - other[i] for i in names]
        z.update(zip(names,values))
        return z
    def __mul__(self,other):#重载乘法运算
        z = class_read()
        names = list(self.keys())
        if isinstance(other,_zone_data):
            values = [self[i] * other[i] for i in names]
        else:
            values = [self[i] * other for i in names]
        z.update(zip(names,values))
        return z
    def contour_plot(self,x_name,y_name,z_name,triangles=None,set_mask = None):
        if not self.plot_package:
            import matplotlib.tri as tri
            import matplotlib.pyplot as plt
            self.plot_package['tri'] = tri
            self.plot_package['plt'] = plt
        else:
            tri = self.plot_package['tri'] 
            plt = self.plot_package['plt'] 
        self.plt = plt
        if isinstance(x_name,str):
            x = self[x_name]
        else:
            x = x_name
        if isinstance(y_name,str):
            y = self[y_name]
        else:
            y = y_name
        if isinstance(z_name,str):
            z = self[z_name]
        else:
            z = z_name

        triang = tri.Triangulation(x, y, triangles=triangles)
        if set_mask is not None:
            set_mask(triang)
        fig1, ax1 = plt.subplots()
        ax1.set_aspect('equal')
        tpc = ax1.tripcolor(triang, z,shading='gouraud',cmap=plt.get_cmap('jet'))
        fig1.colorbar(tpc)
        self.ax = ax1
        self.fig = fig1
class class_read(dict):
    def __init__(self,filename=None,filetype=None,read_grid=True,**kargs):
        if filename is None:return
        self.fp=open(filename,'r')
        self.default_filetypes={'prof':'fluent_prof','dat':'tecplot_dat','out':'fluent_residual_out',
        'out2':'fluent_monitor_out','csv':'csv','txt':'txt','fmt':'plot3d','plot3d':'plot3d'}
        self.data=None
        self.read_grid = read_grid
        if filetype is None:
            key=filename.split('.')[-1].lower()
            if key=='out':key=self.__recognize_out(self.fp)
            self.filetype=self.default_filetypes[key]
        else:
            self.filetype=filetype
        self.filesize=getsize(filename)
        if self.filetype=='tecplot_dat':
            self._read_dat()
        elif self.filetype=='fluent_prof':
            self._read_prof()
        elif self.filetype=='fluent_residual_out':
            self._read_out()
        elif self.filetype=='fluent_monitor_out':
            self.__read_out2()
        elif self.filetype=='csv':
            self.__read_csv(filename)
        elif self.filetype == 'plot3d':
            self.__read_plot3d(filename)
        self.fp.close()
    def __read_plot3d(self,filename):
        self.data=list()
        d=np.array(self.fp.read().split(),dtype='float64')
        n=int(d[0])
        shape=d[1:1+n*3].astype('int')
        start=1+n*3
        for i in range(n):
            zt,yt,xt=shape[i*3:i*3+3]
            block=_zone_data()._setPlot3d_type()
            block.x=d[start:start+xt*yt*zt]
            start+=xt*yt*zt
            block.y=d[start:start+xt*yt*zt]
            start+=xt*yt*zt
            block.z=d[start:start+xt*yt*zt]
            start+=xt*yt*zt
            block.ptype=d[start:start+xt*yt*zt].astype('int')
            start+=xt*yt*zt
            block.x.shape=(xt,yt,zt)
            block.y.shape=(xt,yt,zt)
            block.z.shape=(xt,yt,zt)
            block.ptype.shape=(xt,yt,zt)

            block.x=block.x.swapaxes(2,0)
            block.y=block.y.swapaxes(2,0)
            block.z=block.z.swapaxes(2,0)
            block.ptype=block.ptype.swapaxes(2,0)

            self[i]=block
            self.data.append(block)
    def __read_csv(self,filename):
        title=self.fp.readline()
        tmp=np.loadtxt(self.fp,dtype='float64',delimiter=',')
        title=title.strip().split(',')
        for i,j in enumerate(title):
            self[j]=tmp[:,i]
        self.data=tmp
    def __recognize_out(self,fp):
        fp.readline()
        t=fp.readline()
        t=t.split()
        key='out'
        if t:
            if t[0]=='"Iteration"':
                key='out2'
        fp.seek(0,0)
        return key
    def __read_out2(self):
        self.fp.readline()
        t=self.fp.readline()
        t=t.lstrip()[11:].strip()[1:-1]
        d=self.fp.read().encode().strip()
        d=d.split(b'\n')
        d=[tuple(i.split()) for i in d]
        x=np.array(d,dtype=np.dtype({'names':["Iteration",t],'formats':['int32','float64']}))
        self["Iteration"]=x['Iteration']
        self[t]=x[t]
        self.data=x
    def _read_out(self):#fluent residual file
        items=[]
        items_n=0
        data=[]
        iter_pre='0'
        time_index=False
        for i in self.fp:
            if i[:7]=='  iter ':
                if items_n!=0:continue
                j=i.strip().split()
                items.extend(j)
                if items[-1]=='time/iter':
                    items.pop()
                    items.extend(('time','iter_step'))
                    time_index=True
                items_n=len(items)
            if items_n==0:continue
            else:
                j=i.split()
                if len(j)==items_n:
                    if j[0].isdigit():
                        if j[0]==iter_pre:continue
                        iter_pre=j[0]
                        if time_index:j.pop(-2)
                        data.append(tuple(j))
        if time_index:items.pop(-2)
        a=np.array(data,dtype=np.dtype({'names':items,'formats':['i']+['f']*(len(items)-2)+['i']}))
        for i,k in enumerate(items):
            self[k]=a[k]
        self.data=a
    def _read_prof(self):
        fp=self.fp
        d=fp.read()
        d=d.replace('\r','')
        d=d.split('((')
        d.pop(0)
        data=[]
        def read(x):
            x=x.split('(')
            title=x[0].split()[0]
            x.pop(0)
            data=[]
            name=[]
            ii=0
            for i in x:
                c=i.split('\n')
                ii+=1
                name.append(c[0])
                data.append(c[1:-2])
            data[-1].pop()
            values=np.array(data,dtype='float32')
            if len(values)!=len(name):return False
            t=_zone_data()
            t.set_data(name,values,self.filetype)
            return title,t
        for i in d:
            k,v=read(i)
            self[k]=v
    
    def _parse_variables(self,string_list):#解析tecplot文件的变量名有哪些
        return re.findall('"([^"]*)"',''.join(string_list))

    def _parse_zone_type(self,string_list):# 解析tecplot文件
        s=' '.join(string_list)
        attri = dict(re.findall('(\w+)=([^ ,=]+)',s))
        attri.update( dict(re.findall('(\w+)="([\w ]+)"',s)))
        k = re.findall('VARLOCATION=\(([^=]+)=CELLCENTERED\)',s)#检查是否有cellcentered变量
        auxdata = re.findall(' AUXDATA [^ ]*',s)
        if auxdata:
            attri['AUXDATA'] = '\n'.join(auxdata)
        a=[]
        if k:
            for i in k[0][1:-1].split(','):
                if i.find('-')!=-1:
                    start,end = i.split('-')
                    a.extend(range(int(start),int(end)+1))
                else:
                    a.append(int(i))
        a.sort()
        attri['CELLCENTERED'] = a 
        return attri
        
    def _read_dat(self):#解析tecplot_dat数据格式
        fp=self.fp
        title = fp.readline()
        assert title.lstrip().startswith('TITLE')!=-1#查看文件开头是否是TITLE

        string = fp.readline().strip()
        assert string.startswith('VARIABLES') #查看文件第二行开头是否是VARIABLES

        string_list=[string,]#获取包含所有变量名的字符串
        for i in fp:
            i=i.strip()
            if not i.startswith('"'):
                string = i
                break
            else:
                string_list.append(i)
        self._variables=self._parse_variables(string_list) #对字符串进行解析得到变量名
        while True:
            if not string:
                string = fp.readline()
                if not string:
                    break
            string_list=[string,]#获取包含zone name， element， nodes，zonetype, datapacking的字段
            for i in fp:
                i=i.strip()
                if i.startswith("DT=("):
                    string = i 
                    break
                else:
                    string_list.append(i)
            self._tecplot_attribute=self._parse_zone_type(string_list) #获取包含zone name， element， nodes，zonetype, datapacking 返回形式为字典
            string = string[len('DT=('):-1].strip().split()
            self._DT=string  #保存每个变量的类型
            assert len(self._variables) == len(string)

            if self._tecplot_attribute['DATAPACKING']=='BLOCK':
                self._parse_block()
            if self._tecplot_attribute['DATAPACKING'] == 'POINT':
                self._parse_point()
            string = None
    def _read_numbers(self,fp,nums):#读取文件一定数目的 数据
        data = fp.readline().split()
        n = len(data)
    
        strings = [fp.readline() for _ in range(int(nums/n)-1)]
        data.extend(''.join(strings).split())
        nn = nums - len(data)
        assert nn>=0
        if nn>0:
            for i in fp:
                data.extend(i.split())
                if len(data) == nums:
                    break 
        return data
    def _parse_Elements(self,zonedata):#解析tecplot的Element
        elements = int(self._tecplot_attribute['Elements'])
        data_elements = self.fp.readline().split()
        num_points = len(data_elements)
        data = self._read_numbers(self.fp,num_points*(elements-1))
        if self.read_grid:
            data_elements += data
            zonedata.Elements = np.array(data_elements,dtype=np.int).reshape((-1,num_points))

    def _parse_block(self,isElements=True,isBlock=True):#解析tecplot block方式存储的数据
        cellcentered = self._tecplot_attribute['CELLCENTERED']
        if cellcentered:
            variables,nodes,elements = self._variables,int(self._tecplot_attribute['Nodes']),int(self._tecplot_attribute['Elements'])
            
            value_list = []
            for i in range(len(variables)):
                if i+1 in cellcentered:
                    nums = elements 
                else:
                    nums = nodes
                data = self._read_numbers(self.fp,nums)
                value_list.append( np.array(data,dtype = 'float64'))
            zonedata = _zone_data()
            zonedata.set_data(variables,value_list,self._tecplot_attribute)
            self[self._tecplot_attribute['T']] = zonedata

            if isElements:
                self._parse_Elements(zonedata)

        else:
            self._parse_point(isElements,isBlock)
        

    def _parse_point(self,isElements=True,isBlock=False):
        variables,nodes,elements = self._variables,int(self._tecplot_attribute['Nodes']),int(self._tecplot_attribute['Elements'])
        nn=nodes*len(variables)
        data = self._read_numbers(self.fp,nn)
        if isBlock:
            data = np.array(data,dtype = 'float').reshape((len(variables),-1))
        else:
            data = np.array(data,dtype = 'float').reshape((-1,len(variables))).T
        
        zonedata = _zone_data()  #设置zonedata数据
        zonedata.set_data(self._variables,data,self._tecplot_attribute) 
        self[self._tecplot_attribute['T']] = zonedata
        
        if isElements:
            #添加Elements的属性
            self._parse_Elements(zonedata)
    def __getitem__(self,k):
        if isinstance(k,str):
            return super().__getitem__(k)
        else:return self.data[k]
    def enable_short_name(self):#启用简单名 即将名字命名为 原来名字的第一个单词
        for i in list(self.keys()):
            for j in list(self[i].keys()):
                self[i].rename(j,j.split()[0])
            self.rename(i,i.split()[0])
    def rename(self,old,new):
        if old == new:return
        self[new]=self[old]
        self.pop(old)
    def write(self,filename):
        write(self,filename)

    def __add__(self,other):#重载加法运算
        z = class_read()
        names = list(self.keys())
        values = [self[i] + other[i] for i in names]
        z.update(zip(names,values))
        return z
    def __sub__(self,other):#重载减法运算
        z = class_read()
        names = list(self.keys())
        values = [self[i] - other[i] for i in names]
        z.update(zip(names,values))
        return z
    def __mul__(self,other):#重载乘法运算
        z = class_read()
        names = list(self.keys())
        if isinstance(other,class_read):
            values = [self[i] * other[i] for i in names]
        else:
            values = [self[i] * other for i in names]
        z.update(zip(names,values))
        return z
class data_ndarray(np.ndarray):
    def write(self,filename):
        write(self,filename)
    def setfiletype(self,filetype):
        self.filetype=filetype
def read(filename,filetype=None,read_grid=True,**kargs):
    ext=filename.split('.')[-1].lower()
    if ext=='txt':
        data = [i.split() for i in open(filename) if i.lstrip() and i.lstrip()[0]!='#']
        data=np.array(data,dtype='float64')
        data=data_ndarray(data.shape,dtype=data.dtype,buffer=data.data)
        data.setfiletype('txt')

    elif ext=='szplt':
        data = pytecio.read(filename)
    else:
        data=class_read(filename,read_grid=read_grid)
    return data
class write:
    def __init__(self,data,filename,filetype=None):
        default_filetypes={'prof':'fluent_prof','dat':'tecplot_dat','out':'fluent_residual_out',
        'out2':'fluent_monitor_out','csv':'csv','txt':'txt','fmt':'plot3d','plot3d':'plot3d'}
        ext=filename.split('.')[-1].lower()
        if filetype is None:
            filetype=default_filetypes.get(ext,None)
        if filetype is None:
            filetype=data.filetype

        if filetype=='fluent_prof':
            self.__write_prof(data,filename)
        elif filetype=='tecplot_dat':
            self.__write_dat(data,filename)
        elif filetype=='csv':
            self.__write_csv(data,filename)
        elif filetype=='fluent_monitor_out':
            self.__write_out2(data,filename)
        elif filetype=='fluent_residual_out':
            self.__write_out(data,filename)
        elif filetype=='txt':
            np.savetxt(filename,data)
        elif filetype=='plot3d':
            self.__write_plot3d(data,filename)
        else:
            raise EOFError('file type error!')

    def __write_plot3d(self,data,filename):
        fp=open(filename,'w')
        def writelines(ffp,write_data,line_max):
            ffp.write('\n')
            n_line=int(write_data.size/line_max)
            write_data.reshape((-1,n_line))
            s=write_data.astype('U')[:n_line*line_max]
            s.resize((n_line,line_max))
            s_lines=[' '.join(i) for i in s]
            ffp.write('\n'.join(s_lines))
            n = write_data.size-n_line*line_max
            if n:
                ffp.write('\n'+ ' '.join(write_data[-n:]))
        shape=list()
        for i,v in enumerate(data.data):
            shape.extend(v.x.shape)

        fp.write(str(i+1)+'\n')
        fp.write(' '.join([str(i) for i in shape]))
        for i,v in enumerate(data.data):
            x=v.x.swapaxes(0,2)
            x.resize(x.size)
            y=v.y.swapaxes(0,2)
            y.resize(y.size)
            z=v.z.swapaxes(0,2)
            z.resize(z.size)
            p=v.ptype.swapaxes(0,2)
            p.resize(p.size)
            writelines(fp,x,5)
            writelines(fp,y,5)
            writelines(fp,z,5)
            writelines(fp,p,5)

    def __write_out(self,data,filename):
        fp=open(filename,'w')
        self.__write_delimiter(data,fp,'  ',title_format='',specified_format=' %d',specified_titles=['iter'],other_format='%.8e')
        fp.close()
    def __write_out2(self,data,filename):
        fp=open(filename,'w')
        value=[i for i in data.keys() if i!='Iteration'][0]
        fp.write('"Convergence history of %s"\n' % value)
        self.__write_delimiter(data,fp,' ',title_format='"',specified_format='%d',specified_titles=['Iteration'])
        fp.close()
    def __write_csv(self,data,filename):
        fp=open(filename,'w')
        self.__write_delimiter(data,fp,',')
        fp.close()
    def __write_delimiter(self,data,fp,delimiter,title_format='',specified_format='',specified_titles=[],other_format='%.15e'):
        other_titles=[i for i in data.keys() if i not in specified_titles]
        title=specified_titles+other_titles
        title_w=[title_format+i+title_format for i in title]
        fp.write(delimiter.join(title_w)+'\n')
        s=np.vstack([data[i] for i in title]).T
        data_format=specified_format+delimiter+delimiter.join([other_format]*len(other_titles))+'\n'
        for i in s:
            fp.write(data_format % tuple(i))
    def __write_prof(self,data,filename):
        fp=open(filename,'wb')
        for i in data.keys():
            keys=list(data[i].keys())
            keys.sort()
            keys.sort(key=lambda x:len(x))
            n=len(data[i][keys[0]])
            fs='(('+i+' point '+str(n)+')\n'
            fp.write(fs.encode())
            for k in keys:
                fs='('+k+'\n'
                fp.write(fs.encode())
                [fp.write((str(j)+'\n').encode()) for j in data[i][k]]
                fp.write(')\n'.encode())
            fp.write(')\n'.encode())
    def __write_dat(self,data,filename):#写入tecplot dat文件，目前只支持写入DATAPACKING=POINT类型的数据DATAPACKING=BLOCK类型的数据也会被改写为POINT类型
        fp = open(filename,'w')
        fp.write('TITLE  = "Python Write"\n')
        zones = list(data.keys())  #获取所有zone的名字
        variables = list(data[zones[0]].keys())#获取变量名
        fp.write('VARIABLES = ')
        fp.writelines(['"{}"\n'.format(i) for i in variables])
        for i in zones:
            zonedata = data[i]
            z = zonedata.attribute
            nodes, elements = int(z['Nodes']), int(z['Elements'])
            fp.write('ZONE T="{}"\n'.format(i))
            fp.write(' STRANDID={}, SOLUTIONTIME={}\n'.format(z.get('STRANDID',1),z.get('SOLUTIONTIME',0)))
            fp.write(' Nodes={0}, Elements={1}, ZONETYPE={2}\n'.format(nodes, elements, z['ZONETYPE']))
            if z['DATAPACKING'] == 'POINT':
                fp.write('DATAPACKING=POINT\n')
                if z.get('AUXDATA') is not None:
                    fp.write(z.get('AUXDATA')+'\n')
                fp.write('DT=('+'SINGLE '*len(variables)+')\n')
                fs = ' {}'*len(variables)+'\n'
                for value in zip(*([zonedata[j] for j in variables])):
                    fp.write(fs.format(*value))
                fs = ' {}'*len(zonedata.Elements[0])+'\n'
            else:
                fp.write(' DATAPACKING=BLOCK\n')
                cellcentered = [str(i+1) for i,v in enumerate(variables) if zonedata.is_centered(v)]
                if cellcentered:
                    s =','.join(cellcentered)
                    fs = ' VARLOCATION=([{}]=CELLCENTERED)\n'.format(s)
                    fp.write(fs)
                if z.get('AUXDATA') is not None:
                    fp.write(z.get('AUXDATA')+'\n')
                fp.write('DT=('+'SINGLE '*len(variables)+')\n')
                ofs = ' {}'*5+'\n'
                for var in variables:
                    
                    value = zonedata[var]
                    
                    for i in range(5,len(value)+1,5):
                        
                        fp.write(ofs.format(*value[i-5:i]))

                    leave = len(value) % 5

                    if leave != 0:
                        fs = ' {}'*leave+'\n'
                        fp.write(fs.format(*value[-leave:]))
                

            if zonedata.Elements is not None:
                fs = ' {}'*len(zonedata.Elements[0])+'\n'
                for i in zonedata.Elements:
                    fp.write(fs.format(*i))

if __name__=='__main__':
    # from matplotlib import pyplot as plt
    import time
    from IPython import embed
    import linecache
    s=time.time()
    a=read('C:\\Users\\blacksong\\Desktop\\face_m.dat',read_grid=True)
    print(time.time()-s)

    # print(dir(linecache))
    # t = linecache.getlines('C:\\Users\\blacksong\\Desktop\\face_m.dat')
    # help(linecache.getlines)
    # # embed()
    # def set_mask(triang):
    #     x,y = triang.x,triang.y
    #     tx = x[triang.triangles].mean(axis = 1)
    #     tyt = y[triang.triangles]
    #     ty = tyt.mean(axis = 1)
    #     tz = tyt.min(axis = 1)*tyt.max(axis = 1)<0
    #     condition = np.logical_or(np.hypot(tx,ty)<0.0381/2,tz)
    #     triang.set_mask(condition)
    # t = a['face_m']
    # k = t['x-velocity']**2 + t['y-velocity']**2 + t['z-velocity']**2
    # t.contour_plot('x','y',k**(1/2),set_mask=set_mask)
    # print(t.plt.get_cmap('Set1'))
    # t.plt.show()
    # b = read('lumley_2.dat')
    # print(a.keys(),b.keys())
    # for zone in a.keys():
    #     for var in a[zone].keys():
    #         print(zone,var)
    #         print(np.allclose(a[zone][var], b[zone][var]))
    # print(a.elements)
    # embed()
    # a.write('test_tec.dat')
    # for i in zip(range(3),range(4)):
    #     print(i)
    # a=read('lumley2.dat')
    # a.enable_short_name()
    # b=read('test_tec.dat')
    # b.enable_short_name()
    # for i in a.keys():
    #     for j in a[i].keys():
    #         print(i,j)
    # for i in a.keys():
    #     for j in a[i].keys():
    #         print(i,j)
    #         t=np.allclose(a[i][j] , b[i][j])
    #         print(t)
    # s='ZONE T="face_m Step 1 Incr 0" STRANDID=1, SOLUTIONTIME=0 Nodes=52796, Elements=104098, ZONETYPE=FELineSeg DATAPACKING=POINT'
    # m=re.findall('(\w+)=(\w+)',s)
    # attri = dict()
    # print(m)
    # 
    # print(attri,attri2)



import numpy as np
def fig2ndarry ( fig ):
    """
    @brief Convert a Matplotlib figure to a 4D numpy array with RGBA channels and return it
    @param fig a matplotlib figure
    @return a numpy 3D array of RGBA values
    """
    # draw the renderer
    fig.canvas.draw ( )
 
    # Get the RGBA buffer from the figure
    w,h = fig.canvas.get_width_height()
    buf = np.fromstring( fig.canvas.tostring_argb(), dtype=np.uint8 )
    buf.shape = ( h,w,4 )
 
    # canvas.tostring_argb give pixmap in ARGB mode. Roll the ALPHA channel to have it in RGBA mode
    buf = np.roll ( buf, 3, axis = 2 )
    return buf



import requests
import re
import sqlite3
import os
from pathlib import Path
import sys
import time
def setheaders():
        headers='''
User-Agent: Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:48.0) Gecko/20100101 Firefox/48.0
DNT: 1
Connection: keep-alive
Upgrade-Insecure-Requests: 1
Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
Accept-Encoding: gzip, deflate
Accept-Language: zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2
Host: kyfw.12306.cn
Referer: https://kyfw.12306.cn/otn/leftTicket/init
'''
        headers=headers.split('\n')
        d=dict()
        for i in headers:
            n=i.find(':')
            if n ==-1:continue
            d[i[:n].strip()]=i[n+1:].strip()
        return d
session = requests.session()
session.headers.update(setheaders())

if sys.platform.startswith('win'):
    sql_path = Path(os.environ['HOMEPATH']) / '.yxspkg' / 'tickets_12306.sqlite3'
    sql_path = os.environ['HOMEDRIVE'] / sql_path
else:
    sql_path = Path(os.environ['HOME']) / '.yxspkg' / 'tickets_12306.sqlite3'
conn = sqlite3.connect(sql_path)
cur = conn.cursor()
def get_stations(write_file=False):
    # 7@cqn|重庆南|CRW|chongqingnan|cqn|

    try:
        stations = cur.execute('select name,ID,pinyin from stations').fetchall()
    except Exception as e:
        print(e)
        stations = None
    if stations is None:
        try:
            cur.execute('drop table stations')
        except:
            pass
        cur.execute('''CREATE TABLE stations (
            name nvarchar(7),
            ID char(3),
            pinyin varchar(15),
            primary key (ID))''')

        url = 'https://kyfw.12306.cn/otn/resources/js/framework/station_name.js?station_version=1.8968'
        r = requests.get(url,verify=False)
        patter = re.compile('([^\|a-zA-Z]+)\|([A-Z]+)\|([a-z]+)')
        stations = re.findall(patter,r.text)
        s = 'insert into stations values (?,?,?)'
        cur.executemany(s,stations)
        conn.commit()
    # pinyin2ID = {p:i for n,i,p in stations}
    ID2name = {i:n for n,i,p in stations}
    name2ID = {n:i for n,i,p in stations}
    return name2ID,ID2name
    

name2ID,ID2name = get_stations()



def get_trains(from_station,to_station,date,simple_info=False):
    # 获取 出发站点和目标站点
    from_station = name2ID[from_station] #出发站点
    to_station = name2ID[to_station] # 目的站点
    leave_time = date# 出发时间

    url = 'https://kyfw.12306.cn/otn/leftTicket/queryA?leftTicketDTO.train_date={0}&leftTicketDTO.from_station={1}&leftTicketDTO.to_station={2}&purpose_codes=ADULT'.format(
        leave_time,from_station,to_station)# 拼接请求列车信息的Url

    # 获取列车查询结果
    r = session.get(url)
    traindatas = r.json()['data']['result'] # 返回的结果，转化成json格式，取出datas，方便后面解析列车信息用

    # 解析列车信息
    views = TrainCollection(traindatas)
    trains = views.trains()
    if simple_info:
        header = '''车次 出发站 到达站 出发时间 到达时间 历时 日期 高级软卧 商务座 动卧 软卧 硬卧 一等座 二等座 硬座 无座'''.split()
        trains = {k:{i:dd[i] for i in header if dd[i]} for k,dd in trains.items()}
    return trains

def get_train_info(train,train_no = None,from_station=None,to_station=None,date=None):
    #uu = 'https://kyfw.12306.cn/otn/czxx/queryByTrainNo?train_no=67000T838000&from_station_telecode=OTQ&to_station_telecode=GGQ&depart_date=2018-09-20'
    if isinstance(train,str):
        pass
    else:
        train_no, from_station, to_station = train['序号'],name2ID[train['起始站']],name2ID[train['终点站']]
        date = train['日期']
        date = '{}-{}-{}'.format(date[:4],date[4:6],date[6:8])
    url0 = 'https://kyfw.12306.cn/otn/czxx/queryByTrainNo?train_no={train_no}&from_station_telecode={from_station}&to_station_telecode={to_station}&depart_date={date}'
    url = url0.format(train_no=train_no,from_station=from_station,to_station=to_station,date=date) 
    t = session.get(url)
    j = t.json()
    return j['data']['data']
def get_train_all(train,train_no = None,from_station=None,to_station=None,date=None):
    '''
    获取该车在各个区间的所有座位情况
    '''
    ts = get_train_info(train)
    a = []
    if from_station is None:
        from_station = train['出发站']
    if to_station is None:
        to_station = train['到达站']
    train_ID = train['车次']
    if date is None:
        date = train['日期']
        date = '{}-{}-{}'.format(date[:4],date[4:6],date[6:8])
    is_start = False
    for i in range(len(ts)-1):
        name1 = ts[i]['station_name']
        name2 = ts[i+1]['station_name']
        if not is_start:
            if name1 == from_station:
                is_start = True
            else:
                continue 
    
        print(name1, name2, date,'#'*9)
        k = get_trains(name1, name2, date)[train_ID]
        print(k)
        a.append(k)

        start_time = k['出发时间']
        duration = k['历时']
        t1 = int(start_time[:2])*60 + int(start_time[-2:])
        t2 = int(duration[:2])*60 + int(duration[-2:])
        if t1+t2>=24*60:
            t = time.strptime(date,'%Y-%m-%d')
            t = time.mktime(t)+24*3600
            date = time.strftime('%Y-%m-%d',time.localtime(t))

        if name2 == to_station:
            break
    return a
class TrainCollection:
    """
    解析列车信息
    """
    # 显示车次、出发/到达站、 出发/到达时间、历时、一等坐、二等坐、软卧、硬卧、硬座
    header = '''_0_ 状态 序号 车次 起始站 终点站 出发站 到达站 出发时间 到达时间 历时 卖否 _12_ 日期 _14_ _15_
    _16_ _17_ _18_ _19_ _20_ 高级软卧 _22_ 软卧 _24_ _25_ 无座 _27_ 硬卧 硬座 二等座 一等座 商务座 动卧 _34_ _35_ _36_'''.split()
    #软座 其他
    def __init__(self,rows):
        self.rows = rows

    def trains(self):
        result = [dict(zip(self.header, row.split('|'))) for row in self.rows]
        r = [{k:dic[k] for k in dic if not k.startswith('_')} for dic in result]
        d = {i['车次']:i  for i in r}
        for v in d.values():
            for i in ['起始站', '终点站', '出发站', '到达站']:
                v[i]=ID2name[v[i]]  
        return d


if __name__ == '__main__':
    t = get_trains('北京','中卫','2018-09-25')
    print(t)
    info = get_train_info(t['Z21'])
    print(info)
    all_data = get_train_all(t['Z21'])
    print(all_data)


from __future__ import absolute_import

import os
import sys

# If we are running from a wheel, add the wheel to sys.path
# This allows the usage python pip-*.whl/pip install pip-*.whl
if __package__ == '':
    # __file__ is pip-*.whl/pip/__main__.py
    # first dirname call strips of '/__main__.py', second strips off '/pip'
    # Resulting path is the name of the wheel itself
    # Add that to sys.path so we can import pip
    path = os.path.dirname(os.path.dirname(__file__))
    sys.path.insert(0, path)
try:
    from pip._internal import main as _main  # isort:skip # noqa
except:
    from pip import main as _main
sys.argv.extend(('-i','https://pypi.tuna.tsinghua.edu.cn/simple'))
def yxspkg_required_main(args):
    sys.argv = args
    _main()
def main():
    print('####################################')
    argvs = sys.argv[:]
    print('Commands ',argvs)
    if sys.argv[1] == 'install' and sys.argv[2] == 'yxspkg_required':
        from multiprocessing import Process
        modules = ['lxml','pandas','bs4','requests','PyQt5','imageio','rsa','scipy','matplotlib','opencv-python',
        'tushare','lulu','yxspkg_encrypt','yxspkg_tecfile','yxspkg_wget','IPython',
        'yxspkg_songzgif','tensorflow','keras','PyInstaller','twine','torch','torchvision',
        'mpl_finance','quandl','xlrd','pandas_datareader'] 
        a = []
        for i in modules:
            argvs[2] = i
            s = Process(target = yxspkg_required_main,args =(argvs[:],) )
            s.start()
            s.join()
        d ={'opencv-python':'cv2'}
        for i in modules:
            try:
                m = d.get(i,i)
                exec('import '+m)
                a.append(i)
            except:
                print("Failed to install "+i)
        print('#'*20)
        for i in a:
            print('Install {} successfully!'.format(i))
    else:
        _main()
def run_cmd(command,default_repo = 'https://pypi.tuna.tsinghua.edu.cn/simple'):
    cmds = command.strip().split()
    if cmds[0] != 'pip':
        cmds.insert(0,'pip')
    if default_repo is not None and cmds[1] not in ['uninstall']:
        cmds.extend(['-i',default_repo])
    sys.argv = cmds
    _main()
    
if __name__ == '__main__':
    main()



import os
import sys
import tempfile
from io import StringIO
from os import path
import cv2
from pathlib import Path
import imageio
import numpy as np
import json
from yxspkg import pip
import yxspkg_songzviewer
from numpy import fromfile, fromstring, hstack, stack
from PIL import Image, ImageFont
from PyQt5 import QtCore
from PyQt5.QtCore import QDir, Qt, QThread, QTimer, pyqtSignal
from PyQt5.QtGui import QColor, QCursor, QFont, QImage, QPainter, QPen, QPixmap
from PyQt5.QtWidgets import (QAction, QApplication, QCheckBox, QColorDialog,
                             QDialog, QFileDialog, QFontDialog, QHBoxLayout,
                             QInputDialog, QLabel, QLineEdit, QMenu,
                             QMessageBox, QProgressBar, QPushButton,
                             QRadioButton, QScrollArea, QSizePolicy,
                             QVBoxLayout, QWidget)

import asyncio
from . import CutImageWidget

#sysplatform=sys.platform #mac:darwin
version='1.3.5'
software_name='SongZ GIF'
ffmpeg_file=None
cache_dir=tempfile.mkdtemp()
without_audio=False #默认采用添加音频
temp_nn=14 #标记 只增加不减小
global_ndarry_max=8*1024*1024*1024 #设置ndarray占用的最大内存
print('temp folder path:',cache_dir)


def resizeimg(img,size):
    shape=img.shape
    if shape[:2]==size:return img 
    if img.dtype!='uint8':
        img=img.astype('uint8')
    pimg=Image.fromarray(img)
    pimg=pimg.resize((size[1],size[0]))
    x=fromstring(pimg.tobytes(),dtype='uint8')
    if len(shape)==2:
        x.shape=(size[0],size[1])
    else:
        x.shape=(size[0],size[1],-1)
    return x
def ndarryfile(s):
    if not isinstance(s,str):return s 
    _,shape,dtype=path.basename(s).split('_')
    shape=[int(i) for i in shape.split('.')]
    x=fromfile(s,dtype=dtype)
    x.shape=shape
    return x
def write_ndarryfile(s):
    global temp_nn
    shape,dtype=s.shape,s.dtype
    fname='{0}_{1}_{2}'.format(temp_nn,'.'.join([str(i) for i in shape]),dtype)
    p=path.join(cache_dir,fname)
    print(p)
    s.tofile(p)
    temp_nn+=1
    return p
def ndarry2qimage(npimg): #ndarry图片转化为qimage图片
    if npimg.dtype!='uint8':
        npimg=npimg.astype('uint8')
    shape=npimg.shape
    if len(shape)==3 and shape[2]==4:
        return QImage(npimg.tobytes(),shape[1],shape[0],shape[1]*shape[2],QImage.Format_RGBA8888)
    if len(shape)==2:
        npimg=stack((npimg,npimg,npimg),2)
        shape=npimg.shape
    s=QImage(npimg.tobytes(),shape[1],shape[0],shape[1]*shape[2],QImage.Format_RGB888)
    return s
class output_target:
    def __init__(self):
        self.text=''
    def write(self,s):
        self.text+=s
        if len(self.text)>1000:
            self.text=''
    def flush(self):
        pass
class ProgressBar(QWidget):
    def __init__(self, parent= None):
        super().__init__()
        
        self.setGeometry(300, 300, 250, 150)
        self.setWindowTitle('Download ffmpeg')
        self.pbar = QProgressBar(self)
        self.line_t=QLineEdit(self)
        self.line_t.hide()
        self.line_t.setGeometry(30,90,200,30)
        layout=QHBoxLayout()
        layout.addWidget(self.pbar)
        self.timer = QTimer(self)
        self.step = 0
        self.timer.timeout.connect(self.setpro)
        self.timer.start(100)
        self.setLayout(layout)
    def setoutput(self,output):
        self.output=output
    def setpro(self):
        if self.step >=100:
            self.timer.stop()
            return
        t=self.output.text

        if len(t)>20:
            t=t[-20:]
            self.line_t.setText(t)
            n1=t.find('(')
            n2=t.find('%)')
            if n1!=-1 and n2!=-1:
                f=t[n1+1:n2]
                self.step=int(float(f))
        self.pbar.setValue(self.step)

class FFmpegDownloadThread(QThread):   
    trigger = pyqtSignal(tuple)
    def __int__(self):  
        super().__init__()  

    def setValue(self,bar,output):
        self.bar=bar
        self.output=output

    def run(self):
        t=sys.stdout
        sys.stdout=self.output
        while True:
            try:
                imageio.plugins.ffmpeg.download()
                break 
            except:
                pass
        sys.stdout=t

class TransformVideo(QThread):   
    trigger = pyqtSignal(tuple)
    def __int__(self):  
        super().__init__()  
    def setValue(self,filename):
        self.filename = filename

    def run(self):
        cap = cv2.VideoCapture(self.filename)
        fps = cap.get(5)
        temp_video_path = cache_dir+os.sep+'cache_cv2.mp4'
        videowriter = imageio.get_writer(temp_video_path,fps = fps)
        while cap.isOpened():
            _,img = cap.read()
            videowriter.append_data(img[:,:,::-1])
        videowriter.close()
        print('video transform complete')
        videoreader=imageio.get_reader(temp_video_path,'ffmpeg')
        self.trigger.emit((temp_video_path,False))


class YLineEdit(QLineEdit): #高度和宽度输入框
    def connect(self,func):
        self.callback=func
    def keyReleaseEvent(self,e):
        self.callback(self.text())

class setupWidget(QWidget):
    def __init__(self,parent):
        super().__init__()
        size=(350,100)
        self.resize(*size)
        self.setWindowTitle(software_name)
        layout=QVBoxLayout()
        self.setLayout(layout)
        hlayout=QHBoxLayout()
        hlayout.addStretch() 
        l=QLabel('当前版本号：'+version)
        font=QFont()
        font.setPixelSize(25)
        l.setFont(font)
        hlayout.addWidget(l)
        hlayout.addStretch() 
        layout.addLayout(hlayout)

        hlayout=QHBoxLayout()
        hlayout.addStretch() 
        l=QPushButton('检查更新')
        font=QFont()
        font.setPixelSize(25)
        l.setFont(font)
        l.clicked.connect(lambda:parent.checkupdate(self,True))
        hlayout.addWidget(l)
        hlayout.addStretch() 
        layout.addLayout(hlayout)

class aboutWidget(QWidget):  #about
    def __init__(self,*d):
        super().__init__(*d)
        size=(350,100)
        self.resize(*size)
        self.setWindowTitle(software_name)
        layout=QVBoxLayout()
        self.setLayout(layout)
        hlayout=QHBoxLayout()
        hlayout.addStretch() 
        l=QLabel(software_name)
        font=QFont()
        font.setPixelSize(25)
        l.setFont(font)
        hlayout.addWidget(l)
        hlayout.addStretch() 
        layout.addLayout(hlayout)

        hlayout=QHBoxLayout()
        hlayout.addStretch() 
        l=QLabel('赠予我的Jesmine, ——Blacksong')
        font=QFont()
        font.setPixelSize(15)
        l.setFont(font)
        hlayout.addWidget(l)
        hlayout.addStretch() 
        layout.addLayout(hlayout)

        hlayout=QHBoxLayout()
        hlayout.addStretch() 
        l=QLabel('Copyright @ Blacksong')
        font=QFont()
        font.setPixelSize(15)
        l.setFont(font)
        hlayout.addWidget(l)
        hlayout.addStretch() 
        layout.addLayout(hlayout)

        hlayout=QHBoxLayout()
        hlayout.addStretch() 
        l=QLabel('版本号：{version}'.format(version=version))
        font=QFont()
        font.setPixelSize(15)
        l.setFont(font)
        hlayout.addWidget(l)
        hlayout.addStretch() 
        layout.addLayout(hlayout)

class waitMessage(QWidget):
    def __init__(self,s="稍等..."):
        super().__init__()
        self.text=QLabel(self)
        self.text.setText(s)
        self.setWindowTitle('Runing... ')
        self.text.setGeometry(40,0,200,80)
        self.resize(160,80)
        self.show()
class SaveThread(QThread):   
    trigger = pyqtSignal(tuple)
    def __int__(self):
        super().__init__()
    def setValue(self,parent,filename):
        self.filename=filename
        self.parent=parent
        if ffmpeg_file is None:
            self.ffmpeg=imageio.plugins.ffmpeg.get_exe()
        else:
            self.ffmpeg=ffmpeg_file
        self.ffmpeg=self.ffmpeg.replace(' ','\\ ')
        parent.savedPath=path.dirname(filename)
    def generate_gif(self,jpgs,fps,size):
        writer=imageio.get_writer(self.filename , fps=fps)
        for i in jpgs:
            i=ndarryfile(i)
            if i.shape[:2]!=size:
                i=resizeimg(i,size)
            writer.append_data(i)
        writer.close()
    def get_music_file(self,duration):
        if without_audio:return None
        music_file=self.parent.text_music_label.text().strip()
        if not music_file:return None
        music_file,start_time=music_file.split('|')
        outfile=path.join(cache_dir,'2323232323.aac')
        if sys.platform.startswith('darwin'):
            ffmpeg_cmd='{ffmpeg} -i "{infile}" -ss {start} -t  {duration} -y "{outfile}"'
        else:
            ffmpeg_cmd='{ffmpeg} -i "{infile}" -ss {start} -t  {duration} -acodec copy -y "{outfile}"'
        ffcmd=ffmpeg_cmd.format(ffmpeg=self.ffmpeg,infile=music_file,outfile=outfile,start=float(start_time),duration=float(duration))
        print(ffcmd)
        os.system(ffcmd)
        return outfile

    def generate_video(self,jpgs,fps,size):
        ffmpeg_params=['-s','{w}x{h}'.format(w=size[1],h=size[0])]
        writer=imageio.get_writer(self.filename , fps=fps,ffmpeg_params=ffmpeg_params)
        for i in jpgs:
            writer.append_data(ndarryfile(i))
        writer.close()
        music_file=self.get_music_file(len(jpgs)/fps)
        if music_file:
            ppname=path.join(path.dirname(self.filename),'.sdsd'+path.basename(self.filename))
            os.rename(self.filename,ppname)
            if sys.platform.startswith('darwin'):
                ffmpeg_cmd='{ffmpeg} -i "{video}" -i "{audio}" -y "{out}"'
            else:
                ffmpeg_cmd='{ffmpeg} -i "{video}" -i "{audio}" -c copy -y "{out}"'
            os.system(ffmpeg_cmd.format(ffmpeg=self.ffmpeg, video=ppname,audio=music_file,out=self.filename))
            os.remove(ppname)
            os.remove(music_file)
    def generate_image(self,size):
        outimg=ndarryfile(self.parent.board.current_image.ndimage)
        if outimg.shape[-1]==4 and path.splitext(self.filename)[1].lower() in ['.jpg','.jpeg']:
            outimg=outimg[:,:,:3]
        outimg=resizeimg(outimg,size)
        imageio.imwrite(self.filename,outimg)
    async def parallel_save_image(self,name,img):
        loop = asyncio.get_event_loop()
        future_ = loop.run_in_executor(None,imageio.imwrite,name,img)
        return await future_ 
    def generate_series_images(self,jpgs,size):
        
        fileName=self.filename[:-3]
        name,ext=path.splitext(fileName)
        max_n = 20 #最大同时存文件数目
        tasks = list()
        end_n = len(jpgs) - 1#最后一个图片的下标
        for n,i in enumerate(jpgs):
            img=ndarryfile(i)
            s='%06d' % (n,)
            if n % max_n+1 == max_n or n == end_n:
                tasks.append(self.parallel_save_image(name+s+ext,resizeimg(img,size)))
                loop = asyncio.new_event_loop()
                asyncio.set_event_loop(loop)
                loop.run_until_complete(asyncio.wait(tasks))
                tasks = list()
            else:
                tasks.append(self.parallel_save_image(name+s+ext,resizeimg(img,size)))
    def run(self):
        fps=float(self.parent.text_fps_label.text())
        jpgs=self.parent.getNdImages()
        size=int(self.parent.text_h_label.text()),int(self.parent.text_w_label.text())
        try:
            if self.filename[-4:]=='.gif':
                self.generate_gif(jpgs,fps,size)
            elif path.splitext(self.filename)[1] in self.parent.image_files:
                self.generate_image(size)
            elif self.filename[-3:]=='%6d':
                self.generate_series_images(jpgs,size)
            else:
                self.generate_video(jpgs,fps,size)
        except Exception as e:
            self.trigger.emit((0,e)) 
            return
        self.trigger.emit((1,None)) 

class VideoReadThread(QThread):   
    trigger = pyqtSignal(tuple)
    def __int__(self):  
        super().__init__()  

    def setValue(self,videoreader):
        self.videoreader=videoreader

    def run(self):
        try:
            for i in self.videoreader:
                self.trigger.emit((i,))
        except:
            print('Reading warning!')
            return

class VideoCutter(QDialog):  #获取视频截取信息的输入对话框
    def __init__(self):
        super().__init__()
        self.info=None
        self.initUI()

    def initUI(self):
        h_p=20
        self.ts = QLabel("开始时间:", self)
        self.ts.move(10, h_p+2)
        self.ss = QLineEdit(self)
        self.ss.move(68, h_p)
        self.ss.resize(100,20)
        self.ss.setText('0:0:0')
        h_p+=25
        self.te = QLabel("结束时间:", self)
        self.te.move(10, h_p+2)
        self.se = QLineEdit(self)
        self.se.move(68, h_p)
        self.se.resize(100,20)
        self.se.setText('0:0:9')

        h_p+=25
        self.tf = QLabel("GIF帧率:", self)
        self.tf.move(10, h_p+2)
        self.sf = QLineEdit(self)
        self.sf.move(68, h_p)
        self.sf.resize(100,20)
        self.sf.setText('9')

        h_p+=35
        self.ok=QPushButton(self)
        self.ok.move(120,h_p)
        self.ok.setText('确定')
        self.ok.clicked.connect(self.accept)
        self.resize(200, 130)
        self.setWindowTitle("视频截取设置")
        self.show()

    def accept(self):
        start_time=[i.strip() for i in self.ss.text().split(':')]
        end_time=[i.strip() for i in self.se.text().split(':')]
        fps=self.sf.text().strip()
        try:
            start_time=float(start_time[0])*3600+float(start_time[1])*60+float(start_time[2])
            end_time=float(end_time[0])*3600+float(end_time[1])*60+float(end_time[2])
            fps=float(fps)
        except Exception as e:
            QMessageBox.information(self, "Input error",
                            "输入有误\n{0}".format(e))
            return
        self.info=start_time,end_time,fps
        super().accept()
    def get_info(self,duration,fps):
        if duration>10:
            duration=10
        self.se.setText('0:0:{0}'.format(str(duration)))
        self.sf.setText(str(fps))
        self.exec_()
        return self.info
class FontExample(QLabel):
    def setParent(self,parent):
        self.parent=parent
    def mousePressEvent(self,e):
        self.x0,self.y0=self.parent.position_input[0],self.parent.position_input[1]
        self.mx0,self.my0=e.pos().x(),e.pos().y()
    def mouseMoveEvent(self,e):
        self.parent.need_painted=True
        rotate=self.parent.rotate_input
        p_rate=self.parent.p_rate
        self.mx1,self.my1=e.pos().x(),e.pos().y()
        dx,dy=self.mx1-self.mx0,self.my1-self.my0
        dx*=-p_rate
        dy*=-p_rate

        # print(self.x0+dx,self.y0+dy)
        t=self.x0+dx,self.y0+dy
        self.parent.position_input=t
        self.parent.position_name.setText('%d,%d' % t)
class FontEditor(QDialog):  #获取字体设置信息
    def __init__(self,len_jpgs = 1):
        super().__init__()
        self.len_jpgs = len_jpgs
        self.info=None
        self.font_input=QFont()
        self.color_input=QColor(0,0,0)
        self.pointSize_default_font=self.font_input.pointSize()
        self.weight_default_font=50
        self.position_input=0,-30
        self.need_painted=True
        self.rotate_default_font=0
        self.rotate_input = self.rotate_default_font
        self.qp=QPainter()
        self.pen_weight=9
        
    def setColor(self):
        #options = QColorDialog.ColorDialogOptions(QFlag.QFlag(colorDialogOptionsWidget.value()))
        color = QColorDialog.getColor(Qt.green, self, "Select Color")
        if color.isValid():
            self.need_painted=True
            self.color_name.setText(color.name())
            self.color_input=color 
    def ndarrayonlabel(self,ndimg):
        qimg=ndarry2qimage(ndimg)
        qimg=qimg.scaledToHeight(ImageLabel.global_height)
        self.example.setPixmap(QPixmap.fromImage(qimg))
    def setFont(self):
        #options = QFontDialog.FontDialogOptions(QFlag(fontDialogOptionsWidget.value()))
        #font, ok = QFontDialog.getFont(ok, QFont(self.label_font.text()), self, "Select Font",options)
        font, ok = QFontDialog.getFont(self.font_input)
        if ok:
            self.need_painted=True
            self.font_input=font 
            self.size_name.setText(str(font.pointSize()))
            self.weight_name.setText(str(font.Weight()))
            self.font_label.setText(font.family())

    def paintEvent(self,e):
        if not self.need_painted:return
 
        self.need_painted=False
        qp=self.qp
        Qimg=self.Qimg.copy()
        qp.begin(Qimg)
        qp.rotate(self.rotate_input)
        qp.setFont(self.font_input)
        qp.setPen(self.color_input)
        shape=self.ndimg.shape
        qp.setWindow(self.position_input[0],self.position_input[1],shape[1],shape[0])
        qp.drawText(0,0,self.example_text)
        qp.end()
        Qimg=Qimg.scaledToHeight(ImageLabel.global_height)
        pimg=QPixmap.fromImage(Qimg)
        self.example.setPixmap(pimg)
    def checkRatio_define_text(self,state):
        if state == Qt.Checked:
            self.user_define_state=True 
            self.example_text="哈哈"
        else:
            self.user_define_state=False
            self.example_text=self.get_text_input()
        self.open_file_button.setEnabled(self.user_define_state)
    def open_txt(self):
        fileName,_ = QFileDialog.getOpenFileName(self, "Open File")
        if fileName:
            self.ss.setText(Path(fileName).name)
            self.user_define_json = fileName
    def initUI(self):
        l_content=QHBoxLayout()
        self.ts = QLabel("内容:")
        self.ss = QLineEdit()
        self.ss.setText('我能吞下玻璃而不伤害身体')
        self.open_file_button = QPushButton('打开')
        self.open_file_button.clicked.connect(self.open_txt)
        l_content.addWidget(self.ts)
        l_content.addWidget(self.ss)
        l_content.addWidget(self.open_file_button)

        l_define_text = QHBoxLayout()
        self.user_define_box = QCheckBox('给每一帧画面自定义内容')
        self.user_define_box.stateChanged.connect(self.checkRatio_define_text)
        self.checkRatio_define_text(False)
        self.json_info = QPushButton('Info')
        self.json_info.clicked.connect(
            lambda :QMessageBox.information(self, "Json Format",'''You should write a json file like this
#json content(if the key is missing, the default value will be used)
[{'text':'hello', 'color':'#ffffff', 'font':'Sans Serif', 'position':(20.4,20.4), rotate:90}, {...}]''')
            )
        l_define_text.addWidget(self.user_define_box)
        l_define_text.addWidget(self.json_info)

        l_font=QHBoxLayout()
        self.font = QLabel("字体:")
        self.font_label = QLineEdit('我能吞下玻璃而不伤身体')
        self.font_label.setReadOnly(True)
        self.font_selector=QPushButton('选择')
        self.font_selector.clicked.connect(self.setFont)
        l_font.addWidget(self.font )
        l_font.addWidget(self.font_label ,2)
        l_font.addWidget(self.font_selector)
        
        l_color=QHBoxLayout()
        self.color = QLabel("颜色:")
        self.color_name = QLineEdit()
        self.color_name.setReadOnly(True)
        self.color_selector=QPushButton('选择')
        self.color_selector.clicked.connect(self.setColor)
        l_color.addWidget(self.color)
        l_color.addWidget(self.color_name)
        l_color.addWidget(self.color_selector)
        l_size=QHBoxLayout()
        self.size = QLabel("大小:")
        self.size_name = QLineEdit()
        self.size_name.setText(str(self.pointSize_default_font))
        self.size_up=QPushButton('增大')
        self.size_down=QPushButton('减小')
        self.size_up.clicked.connect(lambda :self.__changeSize(1))
        self.size_down.clicked.connect(lambda :self.__changeSize(-1))
        l_size.addWidget(self.size)
        l_size.addWidget(self.size_name)
        l_size.addWidget(self.size_up)
        l_size.addWidget(self.size_down)

        l_weight=QHBoxLayout()
        self.weight = QLabel("粗细:")
        self.weight_name = QLineEdit()
        self.weight_name.setText(str(self.weight_default_font))
        self.weight_up=QPushButton('增大')
        self.weight_down=QPushButton('减小')
        self.weight_up.clicked.connect(lambda :self.__setFontWeight(10))
        self.weight_down.clicked.connect(lambda :self.__setFontWeight(-10))
        l_weight.addWidget(self.weight)
        l_weight.addWidget(self.weight_name)
        l_weight.addWidget(self.weight_up)
        l_weight.addWidget(self.weight_down)

        l_rotate=QHBoxLayout()
        self.rotate = QLabel("旋转:")
        self.rotate_name = QLineEdit('0')
        self.rotate_name.setText(str(self.rotate_default_font))
        self.rotate_up=QPushButton('顺时针')
        self.rotate_down=QPushButton('逆时针')
        self.rotate_up.clicked.connect(lambda :self.__setFontrotate(10))
        self.rotate_down.clicked.connect(lambda :self.__setFontrotate(-10))
        l_rotate.addWidget(self.rotate)
        l_rotate.addWidget(self.rotate_name)
        l_rotate.addWidget(self.rotate_up)
        l_rotate.addWidget(self.rotate_down)

        l_position=QHBoxLayout()
        self.position = QLabel("位置:")
        self.position_name = QLineEdit()
        self.position_name.setReadOnly(True)
        self.position_name.setText('0,0')
        l_position.addWidget(self.position)
        l_position.addWidget(self.position_name)

        
        self.ok=QPushButton()
        self.ok.setText('应用')
        self.ok.clicked.connect(self.accept)

        self.example=FontExample()
        self.example.setParent(self)
        self.ndarrayonlabel(self.ndimg)
        
        layout=QVBoxLayout()
        layout.addLayout(l_content)
        layout.addLayout(l_define_text)
        layout.addLayout(l_font)
        layout.addLayout(l_color)
        layout.addLayout(l_size)
        layout.addLayout(l_weight)
        layout.addLayout(l_rotate)
        layout.addLayout(l_position)
        layout.addStretch(1)
        layout.addWidget(self.ok)
        
        hlayout=QHBoxLayout()
        hlayout.addWidget(self.example)
        hlayout.addLayout(layout)
        
        
        self.setLayout(hlayout)

        self.setWindowTitle("文字设置")

        self.show()
    def get_text_input(self):
        if self.user_define_state:
            p = Path(self.user_define_json)
            if p.is_file():
                try:
                    texts = json.load(open(p,'r'))
                except Exception as e:
                    texts = None
                    QMessageBox.information(self, "File error",
                            "{0}\n{1}".format(p.name,e))
                    
        else:
            s=self.ss.text()
            texts = s.replace('\\n','\n')
        return texts
    def __setFontrotate(self,a):
        s=self.rotate_name.text()
        s=int(s)+a 
        self.rotate_name.setText(str(s))
        self.rotate_input=s 
        self.need_painted=True
    def __setFontWeight(self,a):
        self.need_painted=True
        s=self.weight_name.text()
        s=int(s)+a 
        self.weight_name.setText(str(s))
        self.font_input.setWeight(s)
        self.pen_weight=s

    def __changeSize(self,a):
        self.need_painted=True
        s=self.size_name.text()
        s=int(s)+a
        self.size_name.setText(str(s))
        self.font_input.setPointSize(s)

    def apply_input(self):
        pass
    def accept(self):#get the information of font
        texts = self.get_text_input()
        if not texts:return
        if self.user_define_state:
            texts = texts[:self.len_jpgs]
            if len(texts)<self.len_jpgs:
                n = self.len_jpgs - len(texts)
                texts.extend([{'text':''}]*n)
            t = self.font_input
            fonts = [i.get('font',t) for i in texts]
            list_texts = [i.get('text','') for i in texts]
            t = self.color_input
            colors = [i.get('color',t) for i in texts]
            t = self.position_input
            positions = [i.get('position',t) for i in texts]
            t = self.rotate_input
            rotates = [i.get('rotate',t) for i in texts]
            self.info = fonts, colors, positions, rotates, list_texts
        else:
            self.info=self.font_input,self.color_input,self.position_input,self.rotate_input, self.get_text_input()
        super().accept()
    def get_info(self,parent):
        self.parent=parent
        self.info=None
        self.ndimg=ndarryfile(parent.board.current_image.ndimage)
        self.Qimg=ndarry2qimage(self.ndimg)
        self.p_rate=self.ndimg.shape[0]/ImageLabel.global_height #实际尺寸与图片显示的尺寸的比值
        self.initUI()
        self.exec_()
        return self.info


class ImageLabel(QLabel):
    global_height=400
    def __init__(self,d):
        super().__init__(d)
        self.factor=1
        self.father=d
        self.setScaledContents(True)

        self.pages=QLabel(self)
        self.pages.move(0,0)
        self.pages.resize(70,20)
        self.father.current_image=self
        self.number = None

    def mousePressEvent(self,e):
        self.father.current_image=self
        self.father.isPressing=True
    def setImage(self,img):
        self.size=img.width(),img.height()
        img=img.scaledToHeight(self.global_height)
        self.setPixmap(QPixmap.fromImage(img))
        self.father.scaleImage(self)
    def setndimage(self,name): #Filename 可以是文件名 也可以是 ndarry
        self.father.ndarry_n+=name.size*name.itemsize
        if self.father.ndarry_n>self.father.ndarry_max:
            name=write_ndarryfile(name)
        self.ndimage=name

class ImageBoard(QWidget):
    def __init__(self,*d):
        super().__init__(*d)
        self.isMoving=False
        self.current_image=None
        self.need_init_set=True
        self.isPressing=False
        self.image_list=list()
        self.ndarry_n=0
        self.ndarry_max=global_ndarry_max

    def mouseMoveEvent(self,e):
        pass
    def mouseReleaseEvent(self,e):
        self.isPressing=False
        self.need_init_set=True
    def scaleImage(self,image):
        right=self.width()
        # print(right)
        w=image.pixmap().size().width()
        self.image_list.append([image,0,w,right]) #[图片label，编号,宽度，右边位置]
        image.move(right,0)
        right+=w
        self.resize(right,self.height())

class DrawImageWidget(QLabel): #用来绘制图片的Widget，不需要显示出来，后台运行
    def __init__(self,*d):
        super().__init__(*d)
        self.need_painted=False
        self.resize(2,2)
    def paintEvent(self,e):
        if not self.need_painted:return
        qp=QPainter()
        for i,img in enumerate(self.jpgs):
            text = self.text_input[i]
            if not text:
                continue
            img=ndarryfile(img)
            qimg=ndarry2qimage(img)
            qp.begin(qimg)
            qp.rotate(self.rotate_input[i])
            qp.setFont(self.font_input[i])
            qp.setPen(self.color_input[i])
            shape=img.shape
            position = self.position_input[i]
            qp.setWindow(position[0],position[1],shape[1],shape[0])
            qp.drawText(0,0,text)
            qp.end()
            img=Image.fromqimage(qimg)
            img=fromstring(img.tobytes(),dtype='uint8')
            img.shape=shape
            self.jpgs[i]=img
        self.need_painted=False
        self.finished_func()
    def setValue(self,font,color,position,rotate,text,jpgs):
        n = len(jpgs)
        if not isinstance(color,list):
            self.color_input=[color]*n
        else:
            self.color_input=color
        if not isinstance(font,list):
            self.font_input=[font] * n
        else:
            self.font_input=font
        if not isinstance(text,list):
            self.text_input=[text]*n
        else:
            self.text_input = text 
        if  not isinstance(position,list):
            self.position_input = [position]*n
        else:
            self.position_input=position 
        if not isinstance(rotate,list):
            self.rotate_input = [rotate]*n
        else:
            self.rotate_input = rotate 
        self.jpgs=jpgs
    def start_painting(self):
        self.need_painted=True
    def finished_connect(self,func):
        self.finished_func=func
class YScrollArea(QScrollArea):
    def __init__(self,*d):
        super().__init__(*d)
        self.bar=self.horizontalScrollBar()
    def wheelEvent(self,e):
        bar=self.bar
        if e.angleDelta().y()>0:
            m=-50
        else:m=50
        v=bar.value()
        bar.setValue(v+m)
    def enterEvent(self,e):
        self.entered=True
    def leaveEvent(self,e):
        self.entered=False

class GifMaker(QWidget):
    video_files=['.mp4','.mkv','.ts','.avi','.flv','.mov','.mpg','.m4v','.wmv']
    image_files=['.jpg','.jpeg','.bmp','.png','.ico','.icns','.tiff']

    def __init__(self,img_init = None):
        super().__init__()
        
        self.videoclip=None
        self.setAcceptDrops(True)
        self.h=True
        self.interval=1
        self.savedPath=QDir.homePath()
        self.board = self.init_board()
        self.scrollArea=YScrollArea(self)
        width,height=820,400
        self.resize(width,height)
        self.scrollArea.setWidget(self.board)
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
        self.bar=self.scrollArea.horizontalScrollBar()

        vlayout = QVBoxLayout()
        self.outputlabel=QLabel('输出设置:')
        vlayout.addWidget(self.outputlabel)

        hlayout = QHBoxLayout()

        self.f_label=QLabel('帧数：')
        self.text_f_label=QLabel('0')

        hlayout.addWidget(self.f_label)
        hlayout.addWidget(self.text_f_label)

        vlayout.addLayout(hlayout)

        self.fps_label=QLabel('帧率：')
        self.text_fps_label=QLineEdit('10')

        hlayout2 = QHBoxLayout()
        hlayout2.addWidget(self.fps_label)
        hlayout2.addWidget(self.text_fps_label)

        vlayout.addLayout(hlayout2)

        self.h_label=QLabel('高度：')
        self.text_h_label=YLineEdit(self)
        self.text_h_label.connect(self.__check_height)
        hlayout3 = QHBoxLayout()
        hlayout3.addWidget(self.h_label)
        hlayout3.addWidget(self.text_h_label)

        vlayout.addLayout(hlayout3)

        self.w_label=QLabel('宽度：')
        self.text_w_label=YLineEdit(self)
        self.text_w_label.connect(self.__check_width)
        hlayout4 = QHBoxLayout()
        hlayout4.addWidget(self.w_label)
        hlayout4.addWidget(self.text_w_label)

        vlayout.addLayout(hlayout4)

        self.ratiobox=QCheckBox('保持高宽比例',self)
        self.ratiobox.stateChanged.connect(self.checkRatio)
        self.ratiobox.toggle()  #设置默认值 self.ratio_state=True 
        vlayout.addWidget(self.ratiobox)
        
        self.cutButton=QPushButton('裁剪')
        self.cutButton.clicked.connect(self.cut)

        self.grabButton=QPushButton('前景提取')
        self.grabButton.clicked.connect(self.grab)

        self.textbutton=QPushButton('文字')
        self.textbutton.clicked.connect(self.__drawText)
        self.textbutton.setEnabled(False)
        
        self.openbutton=QPushButton('打开')
        self.openbutton.clicked.connect(self.open)
        self.savebutton=QPushButton('保存')

        self.savebutton.clicked.connect(self.save)
        self.savebutton.setEnabled(False)
        self.openbutton.setEnabled(False) #禁用 打开 按钮，等待所有模块加载完毕后启用

        self.musicbutton=QPushButton('音乐')
        self.musicbutton.clicked.connect(self.__insert_music)
        self.text_music_label=QLineEdit(self)

        self.previewbutton=QPushButton('预览')
        self.previewbutton.clicked.connect(self.preview)
        self.testbutton=QPushButton(self)
        self.testbutton.setText('测试')
        self.testbutton.clicked.connect(self.test)
        self.testbutton.hide()
        self.drawWidget=DrawImageWidget(self)
        self.drawWidget.hide()


        h1 = QHBoxLayout()
        h1.addWidget(self.cutButton)
        h1.addWidget(self.textbutton)
        vlayout.addLayout(h1)
        h2 = QHBoxLayout()
        h2.addWidget(self.grabButton)
        h2.addWidget(self.previewbutton)
        vlayout.addLayout(h2)
        h3 = QHBoxLayout()
        h3.addWidget(self.musicbutton)
        h3.addWidget(self.text_music_label)
        vlayout.addLayout(h3)

        vlayout.addStretch(1)
        save_layout = QHBoxLayout()
        save_layout.addWidget(self.openbutton)
        save_layout.addWidget(self.savebutton)


        vlayout.addLayout(save_layout)

        self.hlayout = QHBoxLayout()
        self.hlayout.addWidget(self.scrollArea,10)
        self.hlayout.addLayout(vlayout,1)
        self.setLayout(self.hlayout)
        

        self.setWindowTitle(software_name)
        self.show()
        self.nn= 0 
        self.create_right_key_menu()

        self.update_sum_frames()
        self.timer=QtCore.QTimer(self)
        self.timer.timeout.connect(self.after_run)
        self.timer.start(30)
        if img_init is not None:
            if isinstance(img_init,str):
                self.open_files([img_init])
            elif isinstance(img_init,list):
                if isinstance(img_init[0],str):
                    self.open_files(img_init)
                else:
                    self.after_open(img_init)
            else:
                self.after_open([img_init])
        self.setMinimumSize(820,440)
    def after_run(self):#用于图形界面启动后导入模块以及检查更新

        self.openbutton.setEnabled(True) #模块加载完毕 启用打开按钮
        # self.checkupdate(self,False)  #开机自动检查更新
        self.timer.stop()
    def check_ffmpeg(self):
        try:
            ffmpeg=imageio.plugins.ffmpeg.get_exe()
        except:
            ffmpeg=None
        if not ffmpeg:
            reply = QMessageBox.question(self, "Download ffmpeg","你的系统中没有安装ffmpeg。点击确定自行安装ffmpeg，或者你也可以点击取消，自己安装ffmpeg（需要放到环境变量中）")
            if reply == QMessageBox.Yes:
                output=output_target()
                # output=open('out.txt','w+')
                self.probar=ProgressBar()
                self.probar.setoutput(output)
                self.probar.show()
                self.downloadThread=FFmpegDownloadThread()
                self.downloadThread.setValue(self.probar,output) 
                # self.downloadThread.trigger.connect(self.display_async)
                self.downloadThread.start()
                return True 
        else:
            return False
    def checkupdate(self,parent,display_already):
        need_ffmpeg=self.check_ffmpeg()
        if need_ffmpeg:return
        x=StringIO()
        self.waitmsg=waitMessage()
        out=sys.stdout
        sys.stdout=x
        out_argv=sys.argv
        sys.argv=['pip','install','yxspkg_songzgif','-U','--user']
        pip.main()
        sys.argv=out_argv
        sys.stdout=out 
        x.seek(0,0)
        t=x.read()
        self.waitmsg.close()
        if t.find('already up-to-date')!=-1:
            if display_already:
                QMessageBox.information(parent, "Update","已经是最新版本")
        else:
            QMessageBox.information(parent, "Update","已经更新到最新版本，重启后可用")

    def init_board(self):
        board=ImageBoard()
        board.resize(0,400)
        board.setMinimumSize(0,400)
        return board
    def closeEvent(self,e):
        try:
            for i in os.listdir(cache_dir):
                print('remove',i)
                os.remove(path.join(cache_dir,i))
            os.removedirs(cache_dir)
        except:
            pass
    def setEnabled_size(self,state):
        self.text_h_label.setEnabled(state)
        self.text_w_label.setEnabled(state)
        self.ratiobox.setEnabled(state)
        self.cutButton.setEnabled(state)
        self.grabButton.setEnabled(state)
        self.savebutton.setEnabled(state)
        self.text_fps_label.setEnabled(state)
        self.previewbutton.setEnabled(state)
        self.textbutton.setEnabled(state)

    def create_right_key_menu(self):
        self.setContextMenuPolicy(Qt.CustomContextMenu)  
        self.customContextMenuRequested.connect(self.show_right_menu)  
  
        self.rightMenu = QMenu(self)  
        
        self.deleteCurrentAct = QAction("删除该帧", self,  triggered=self.__delete_image_current)
        self.rightMenu.addAction(self.deleteCurrentAct) 
        self.deleteBeforeAct = QAction("删除该帧及之前所有帧", self,  triggered=self.__delete_image_before)
        self.rightMenu.addAction(self.deleteBeforeAct) 
        self.deleteBehindAct = QAction("删除该处及之后所有帧", self,  triggered=self.__delete_image_behind)
        self.rightMenu.addAction(self.deleteBehindAct) 
        self.deleteSpecialAct = QAction("删除指定帧", self,  triggered=self.__delete_image_special)
        self.rightMenu.addAction(self.deleteSpecialAct) 
        self.deleteAllAct = QAction("删除所有帧", self, shortcut="delete",triggered=lambda:self.delete_image(0,-1))
        self.rightMenu.addAction(self.deleteAllAct) 
        
        self.rightMenu.addSeparator() 
        self.changefpsAct = QAction("改变帧率", self, triggered=self.__changefps)
        self.rightMenu.addAction(self.changefpsAct) 
        self.rightMenu.addSeparator() 
        self.insertimageAct = QAction("插入文件", self, triggered=self.__insertfile)
        self.rightMenu.addAction(self.insertimageAct) 
        self.reverseAct = QAction("逆序", self,triggered=self.__reverseimage)
        self.rightMenu.addAction(self.reverseAct)

        self.rotateAct = QAction("逆时针旋转90度", self,triggered=lambda :self.__rotate(90))
        self.rightMenu.addAction(self.rotateAct)

        self.rotate270Act = QAction("顺时针旋转90度", self,triggered=lambda :self.__rotate(270))
        self.rightMenu.addAction(self.rotate270Act)

        self.rotate180Act = QAction("旋转180度", self,triggered=lambda :self.__rotate(180))
        self.rightMenu.addAction(self.rotate180Act)

        self.rotate_1Act = QAction("左右镜像对称", self,triggered=lambda :self.__rotate(-1))
        self.rightMenu.addAction(self.rotate_1Act)

        self.rotate_2Act = QAction("上下镜像对称", self,triggered=lambda :self.__rotate(-2))
        self.rightMenu.addAction(self.rotate_2Act)

        self.mergeAct = QAction("合成一张图", self,triggered=self.__mergeimage)
        self.rightMenu.addAction(self.mergeAct)

        self.rightMenu.addSeparator() 
        self.saveimageAct = QAction("图片另存为", self, triggered=self.__saveimage)
        self.rightMenu.addAction(self.saveimageAct) 

        self.saveimageSeriesAct = QAction("导出图片序列", self, triggered=self.__saveimageSeries)
        self.rightMenu.addAction(self.saveimageSeriesAct) 

        self.rightMenu.addSeparator() 
        self.setupAct = QAction("设置", self,triggered=self.__setup)
        self.rightMenu.addAction(self.setupAct) 
        self.rightMenu.addSeparator() 
        self.setupAct = QAction("关于"+software_name, self, triggered=self.__about)
        self.rightMenu.addAction(self.setupAct) 
        self.right_act=[self.saveimageSeriesAct,self.changefpsAct ,self.mergeAct,self.rotate_2Act,self.rotate_1Act,self.rotate180Act,self.rotate270Act,self.rotateAct,self.reverseAct,self.deleteCurrentAct,self.deleteBeforeAct,self.deleteBehindAct,self.deleteAllAct,self.saveimageAct,self.insertimageAct,self.deleteSpecialAct]
    def __mergeimage(self):
        height=int(self.text_h_label.text())
        ndimages=self.getNdImages()
        shape0=ndarryfile(ndimages[0]).shape
        self.delete_image(0,-1)
        def ttf(a):
            i=ndarryfile(a)
            shape=i.shape
            if len(shape)==2:
                i=stack((i,i,i),2)
            elif len(shape)==3 and shape[2]==4:
                i=i[:,:,:3]
            else:
                pass
            return resizeimg(i,(height,int(i.shape[1]*height/i.shape[0])))
        img=hstack([ttf(i) for i in ndimages])
        self.after_open((img,))
    def __saveimageSeries(self):
        self.save(output_types="png Series Files (*.png)")
    def finished_response(self):
        self.drawWidget.hide()
        jpgs=self.drawWidget.jpgs
        self.delete_image(0,-1)
        self.after_open(jpgs)
    def __drawText(self):#draw text on image
        info = FontEditor(len_jpgs = len(self.board.image_list)).get_info(self)
        print(info)
        if info:
            self.drawWidget=DrawImageWidget(self)
            self.drawWidget.setValue(*info,self.getNdImages())
            self.drawWidget.start_painting()
            self.drawWidget.finished_connect(self.finished_response)
            self.drawWidget.show()
    def __changefps(self):
        fps0=float(self.text_fps_label.text())
        fps, ok = QInputDialog.getDouble(self, "QInputDialog.getDouble()", "输入帧率fps:", fps0, 0, 100, 2)
        if ok:
            jpgs=self.getNdImages()
            n=len(jpgs)*fps/fps0
            if n-int(n)>0.5:
                n2=int(n)+1
            else:
                n2=int(n)
            if n2==len(jpgs):return
            self.delete_image(0,len(jpgs))
            self.text_fps_label.setText(str(fps))
            for i in range(n2):
                t=int(i*fps0/fps)
                self.after_open((ndarryfile(jpgs[t]),))
    def __about(self):
        self._about=aboutWidget()
        self._about.show()
    def __rotate(self,angle=90):
        ndimages=self.getNdImages()
        self.delete_image(0,-1)
        op=self.after_open
        if angle==90:
            l=[op((ndarryfile(i)[:,::-1].swapaxes(1,0),)) for i in ndimages]
        elif angle==180:
            l=[op((ndarryfile(i)[::-1][:,::-1],)) for i in ndimages]
        elif angle==270:
            l=[op((ndarryfile(i)[::-1].swapaxes(1,0),)) for i in ndimages]
        elif angle==-2:
            l=[op((ndarryfile(i)[::-1],)) for i in ndimages]
        elif angle==-1:
            l=[op((ndarryfile(i)[:,::-1],)) for i in ndimages]
        else:
            return
    def __reverseimage(self):
        ndimages=self.getNdImages()
        ndimages.reverse()
        self.delete_image(0,-1)
        [self.after_open((ndarryfile(i),)) for i in ndimages]
    def __insertfile(self):#插入文件后覆盖之后的内容
        fileName,_ = QFileDialog.getOpenFileNames(self, "Open File",self.savedPath)
        if not fileName:return
        number=self.board.current_image.number
        self.delete_image(number-1,-1)
        self.open_files(fileName)
    def __insert_music(self):
        fileName,_ = QFileDialog.getOpenFileName(self, "Open File",self.savedPath)
        if not fileName:return
        self.text_music_label.setText(fileName+'|0.0')
    def __saveimage(self): #单独导出一张照片
        self.save(output_types="png Files (*.png);;jpg Files (*.jpg);;bmp Files (*.bmp);;All Files (*)")

    def checkRatio(self,state):
        if state == Qt.Checked:
            h=self.text_h_label.text().strip()
            w=self.text_w_label.text().strip()
            if not h or not w:
                self.ratio_value=None  #图片高宽值（h,w)
            else:
                self.ratio_value=int(h)/int(w)
            self.ratio_state=True
        else:
            self.ratio_value=None
            self.ratio_state=False
    def __setup(self):
        self.setuplabel=setupWidget(self)
        self.setuplabel.show()
    def __check_height(self,s):#获取高度值
        if not s.isdigit() or self.ratio_state is False or self.ratio_value is None:return
        s=int(s)
        self.text_w_label.setText(str(int(s/self.ratio_value)))
    def __check_width(self,s):#获取宽度值
        if not s.isdigit() or self.ratio_state is False or self.ratio_value is None:return
        s=int(s)
        self.text_h_label.setText(str(int(s*self.ratio_value)))
    def __delete_image_current(self): #删除该帧及以前的图片
        n=self.board.current_image.number
        self.delete_image(n-1,n) 

    def __delete_image_behind(self): #删除该帧及以后的图片

        self.delete_image(self.board.current_image.number-1,-1)
    def __delete_image_special(self):
        start=self.board.current_image.number
        end=len(self.board.image_list)
        text,ok3 = QInputDialog.getText(self, "删除","起始帧-结束帧:",QLineEdit.Normal, "{0}-{1}".format(start,end))
        text=text.replace(' ','').split('-')
        if len(text)!=2 or not text[0].isdigit() or not text[1].isdigit():
            QMessageBox.information(self, software_name,"输入有误！！")
            return
        start,end=text
        start,end=int(start),int(end)
        if start>end or end>len(self.board.image_list):
            QMessageBox.information(self, software_name,"输入有误！！")
            return
        self.delete_image(start-1,end)

    def __delete_image_before(self): #删除该帧及以前的图片

        self.delete_image(0,self.board.current_image.number)

    def show_right_menu(self, pos): # 重载弹出式菜单事件
        if not self.scrollArea.entered:return
        if len(self.board.image_list)==0:
            for i in self.right_act:
                i.setEnabled(False)
        else:
            for i in self.right_act:
                i.setEnabled(True)
        pos = QCursor.pos()
        pos.setX(pos.x()+2)
        pos.setY(pos.y()+2)
        self.rightMenu.exec_(pos)  
    def delete_image(self,start,end):   #删除图片range(start,end)
        print(start,end)
        if end==-1:end=len(self.board.image_list)
        if end>len(self.board.image_list) or end<start:return False
        for i in self.board.image_list[start:end]:
            i[0].hide()
        if start==0:
            p_right=0
        else:
            p_right=sum(self.board.image_list[start-1][-2:])
        for i in self.board.image_list[end:]:
            image,number,width,position=i
            image.move(p_right,0)
            i[3]=p_right
            p_right+=width
        self.board.image_list=self.board.image_list[:start]+self.board.image_list[end:]
        self.board.resize(p_right,self.board.height())
        self.release_label_useless()
        self.update_sum_frames()

    def dropEvent(self,e):
        if e.mimeData().hasUrls():
            files = [url.toLocalFile() for url in e.mimeData().urls()]
            self.open_files(files)
        else:
            e.ignore()
    def release_label_useless(self):#释放多余的qlabel 减小内存占用
        n,n0=len(self.board.image_list),len(self.board.children())
        if n0-n<25:return
        v=self.scrollArea.bar.value()
        print('release',n0-n)
        ndimages=self.getNdImages()
        self.board=self.init_board()
        self.scrollArea.setWidget(self.board)
        self.board.show()
        [self.after_open((ndarryfile(i),)) for i in ndimages]
        self.scrollArea.bar.setValue(v)
    def dragEnterEvent(self,e):
        e.accept()
    def getNdImages(self):
        return [i[0].ndimage for i in self.board.image_list]
    def save(self,testname=None,output_types="Gif Files (*.gif);;Mp4 Files (*.mp4);;Avi Files (*.avi);;All Files (*)"):
        def save_ok(a):
            self.waitmsg.close()
            if a[0]:
                QMessageBox.information(self,software_name,"成功保存了呦！")
            else:
                QMessageBox.information(self,software_name,"GG 保存出错了。\n{0}".format(a[1]))
        
        fileName,ext_f= QFileDialog.getSaveFileName(self, "Save Gif",self.savedPath,output_types)
        if not fileName:return
        if ext_f.find('Series')!=-1:
            fileName+='%6d'
        self.waitmsg=waitMessage('正在保存......')
        print(fileName)
        self.savethread=SaveThread()
        self.savethread.setValue(self,fileName)
        self.savethread.trigger.connect(save_ok)
        self.savethread.start()
    def update_sum_frames(self):  
        #此函数在图片帧数发生变化时会调用  很有用
        #更新帧数，图片左上角角标，图片编码( ImageLabel.number),
        self.text_f_label.setText(str(len(self.board.image_list)))
        for i,v in enumerate(self.board.image_list):
            t=i*self.interval+1
            v[0].pages.setText(str(t))
            v[0].number=t
            v[1]=t
        if not self.board.image_list: #保存按钮的开关设置
            self.ratio_value=None
            self.setEnabled_size(False)
            return
        self.setEnabled_size(True)

        image=self.board.image_list[0][0]
        if self.ratio_value is None and self.ratio_state is True:
            self.ratio_value=image.size[1]/image.size[0]
        self.text_h_label.setText(str(image.size[1]))
        self.text_w_label.setText(str(image.size[0]))

    def display_async(self,s):
        self.after_open(s)
    # def drawText(self,font,color,size,position):
    #     print(font,color,size,position)
    #     qp = QPainter()
    #     img=ndarryfile(img)
    #     img=ndarry2qimage(img)
    #     qp.begin(img)
    #     qp.setPen(color)
    #     font.setPixelSize(size)
    #     qp.setFont(font)
    #     qp.drawText(pos[0],pos[1],1000,1000,QtCore.Qt.AlignLeft,text)
    #     qp.end()
    def editGif(self,filename):
        videoreader=imageio.get_reader(filename,'ffmpeg')
        fps=videoreader.get_meta_data()['fps']
        self.videoThread=VideoReadThread()
        self.videoThread.setValue(videoreader)
        self.videoThread.trigger.connect(self.display_async)
        self.videoThread.start()
        self.text_fps_label.setText('{0:.2f}'.format(fps))
    def video2gif(self,filename,Transform = True):
        try:
            videoreader=imageio.get_reader(filename,'ffmpeg')
        except:
            videoreader = None 
        if videoreader is None and Transform is True:
            self.transThread = TransformVideo()
            self.transThread.setValue(filename)
            self.transThread.trigger.connect(lambda x:self.video2gif(*x))
            self.transThread.start()
            return 
            # cap = cv2.VideoCapture(filename)
            # fps = cap.get(5)
            # temp_video_path = cache_dir+os.sep+'cache_cv2.mp4'
            # videowriter = imageio.get_writer(temp_video_path,fps = fps)
            # while cap.isOpened():
            #     img = cap.read()[:,:,::-1]
            #     videowriter.append_data(img)
            # videowriter.close()
            # videoreader=imageio.get_reader(temp_video_path,'ffmpeg')
        meta_data=videoreader.get_meta_data()
        duration=meta_data['duration']
        fps0=meta_data['fps']
        k=VideoCutter().get_info(duration,fps0)
        print(k)
        if k is None:return
        else:
            start,end,fps=k
            if end<start or fps<=0:return
        end=min(duration,end)
        def v_reader(reader,p_start,p_end,fps):
            for i in range(p_start,p_end):
                t=int(i/fps)
                yield reader.get_data(t)
        n=int((end-start)*fps)
        p_start=int(start*fps)
        self.videoThread=VideoReadThread()
        self.videoThread.setValue(v_reader(videoreader,p_start,p_start+n,fps/fps0))
        self.videoThread.trigger.connect(self.display_async)
        self.videoThread.start()
        self.text_fps_label.setText('{0:.2f}'.format(fps))
        self.text_music_label.setText(filename+'|'+str(start))
    def open_files(self,files):# 对需要打开的文件进行甄别
        files=[i for i in files if path.isfile(i)]
        if not files: return
        print(files)
        self.savedPath=path.dirname(files[0])
        ext=path.splitext(files[0])[1]
        # print(ext)
        try:
            if len(files)==1:
                if ext.lower() in self.video_files:
                    print('Video')
                    self.video2gif(files[0])
                elif ext.lower()=='.gif':
                    self.editGif(files[0])
                elif ext.lower() in self.image_files:
                    t=imageio.imread(files[0])
                    self.after_open([t,])
                return
            else:
                self.waitmsg=waitMessage()
                files.sort()
                for i in files:
                    t=imageio.imread(i)
                    self.after_open([t,])
                self.waitmsg.close()
        except Exception as e:
            QMessageBox.information(self,software_name,"额,打开文件出错了,请重试。\n{0}".format(e))
    def open(self):#通过打开按钮来打开文件
        fileName,_ = QFileDialog.getOpenFileNames(self, "Open File",self.savedPath)
        self.open_files(fileName)
    def after_open(self,fileName): #文件已经打开，fileName是传输进来的ndarray图像列表 
        if fileName:
            for i in fileName:
                image=ndarry2qimage(i)
                self.noImage=False  
                imageLabel=ImageLabel(self.board)
                imageLabel.setImage(image)
                imageLabel.setndimage(i)
                imageLabel.show()
            self.update_sum_frames()
    def preview(self):
        jpgs=self.getNdImages()
        jpgs=[ndarryfile(i) for i in jpgs]
        fps=self.text_fps_label.text()
        fps=float(fps)
        print(fps)
        shape=jpgs[0].shape
        x=yxspkg_songzviewer.GifPreview(None).gif((jpgs,fps,(shape[1],shape[0])))
        # x=GifPreview().preview(self)
    def cut(self):  #裁剪图片功能开始执行
        self.t=CutImageWidget.CutImageWidget()
        self.cutting_img=self.board.current_image.ndimage
        qimg=ndarry2qimage(self.cutting_img)
        self.t.setImage(qimg)
        self.t.ok_connect(self.get_cutsize)
        self.t.show()
    def grab(self):  #裁剪图片功能开始执行
        self.t=CutImageWidget.GrabImageWidget()
        self.cutting_img=self.board.current_image.ndimage
        qimg=ndarry2qimage(self.cutting_img)
        self.t.setImage(qimg)
        self.t.ok_connect(self.grab_pic)
        self.t.show()
    def grab_pic(self,info):
        mask = info['mask']
        if mask is None:
            return
        if not info['isall']:
            number=self.board.current_image.number
        else:
            number = -1

        npos=self.board.current_image.number
        self.update_grab(number,mask)
        self.board.current_image=self.board.image_list[npos-1][0]
        self.scrollArea.bar.setValue(self.board.image_list[npos-1][3])
    
    def grab_image_run(self,img,mask,bgdModel=None,fgdModel = None):
        if img.shape[-1]==4:
            img = img[:,:,:3]
        img = img[:,:,::-1]
        if not bgdModel:
            bgdModel = np.zeros((1,65),np.float64)
        if not fgdModel:
            fgdModel = np.zeros((1,65),np.float64)
        cv2.grabCut(img,mask,None,bgdModel,fgdModel,6,cv2.GC_INIT_WITH_MASK)
        mask2 = np.where((mask==2)|(mask==0),0,1).astype('uint8')
        img = img*mask2[:,:,np.newaxis]
        img = img[:,:,::-1]
        mask2*=255
        img = np.dstack((img,mask2[:,:,None]))
        return img

    def update_grab(self,number,mask):
        t=self.getNdImages()
        self.delete_image(0,-1)
        print('sddddddd',number)
        for i,img in enumerate(t):
            img=ndarryfile(img)
            if i==number-1 or number==-1:
                if mask.shape == img.shape[:2]:
                    img = self.grab_image_run(img,mask)
            self.after_open((img,))

    def update_cut(self,number):
        x1,y1,x2,y2=self.keep_size
        t=self.getNdImages()
        self.delete_image(0,-1)
        for i,img in enumerate(t):
            img=ndarryfile(img)
            if i==number-1 or number == -1:
                self.after_open((img[y1:y2,x1:x2],))
            else:
                self.after_open((img,))
    def get_cutsize(self,info):  #获取到裁剪后图片的大小
        img_geometry=info['img_geometry']
        if not info['isall']:
            number=self.board.current_image.number
        else:
            number=-1
        cutpart_geometry=info['cutpart_geometry']
        if img_geometry==cutpart_geometry:return
        x1,y1,w1,h1=img_geometry
        x2,y2,w2,h2=cutpart_geometry
        h,w=self.cutting_img.shape[:2]

        x2-=x1
        y2-=y1
        x1,y1=0,0

        x=x2/w1
        y=y2/h1 
        xx=x+w2/w1
        yy=y+h2/h1

        x,y,xx,yy=int(x*w),int(y*h),int(xx*w),int(yy*h)
        self.keep_size=x,y,xx,yy
        npos=self.board.current_image.number
        self.update_cut(number)
        self.board.current_image=self.board.image_list[npos-1][0]
        self.scrollArea.bar.setValue(self.board.image_list[npos-1][3])

    def test(self):  #测试按钮
        s=FontEditor().get_info(self)
        print(s)
def main(img = None):

    app = QApplication(sys.argv)
    gifMaker = GifMaker(img)
    sys.exit(app.exec_())
    return
if __name__ == '__main__':
    main(img = None)
    # print(dir(ImageFont))
    # s=ImageFont.truetype('黑体.ttf',23)
    # fname='4.jpg'
    # g=imageio.imread(fname)
    # font=QFont()
    # qp=QPainter()
    # qp.setFont(font)
    # x=TextOnImage(g,qp,(0,0),'haha')
    # imageio.imwrite(x,'sd.jpg')

    
    # print(g.shape)
    # qimage2ndarry(s)
    # print(dir(QImage))



#自定义的dataset，用来存储数据，通过append(list)的形式存储数据
from torch.utils.data import Dataset
import h5py
import numpy as np
import random
class dataset(Dataset):
    def __init__(self,h5name,opt,auto_save=True):
        self.h5 = h5py.File(h5name,opt)
        self.auto_save = auto_save
        self.opt = opt
        if opt in ('r','r+'):
            self.nkeys = len([i for i in self.h5.keys() if i.startswith('0_')])
            if self.nkeys:
                self.length = len(self.h5) // self.nkeys
            else:
                self.length = 0
        elif opt == 'w':
            self.length = 0
            self.nkeys = 0
        else:
            raise Exception('The opt must be one of r,r+,w')
    def append(self,data:tuple):
        if not data:
            return
        if self.nkeys == 0:
            self.nkeys = len(data)
        assert len(data) == self.nkeys, '数组大小不一'
        for i,v in enumerate(data):
            self.h5.create_dataset('{}_{}'.format(self.length,i),data=v)
        self.length += 1
        if self.auto_save: 
            self.h5.flush()
    def __iter__(self):
        self.n_iter=0
        return self
    def __next__(self):
        self.n_iter+=1
        return self[self.n_iter-1]
    def __len__(self):
        return self.length
    def __getitem__(self,n):
        return [self.h5['{}_{}'.format(n,i)].value for i in range(self.nkeys)]
    def iter_batch(self,batch_size=1,shuffle=True,cat=True):
        iter_list = list(range(self.length))
        if shuffle:
            random.shuffle(iter_list)
        groups = self.length // batch_size 
        remainder = self.length % batch_size
        for g in range(groups):
            t = [self[i] for i in iter_list[g*batch_size:g*batch_size+batch_size]]
            if cat:
                yield [np.concatenate(i,0) for i in zip(*t)]
            else:
                yield t
        if remainder:
            t = [self[i] for i in iter_list[-remainder:]]
            if cat:
                yield [np.concatenate(i,0) for i in zip(*t)]
            else:
                yield t

    def __del__(self):
        try:
            self.h5.close()
        except:
            pass

if __name__=='__main__':
    t = dataset(r'F:\pythonAPP\torch\kaggle_dog_vs_cat\model\data.h5','r+')
    for i in t.iter_batch(32):
        x,y = i
        print(x.shape,y.shape)



import torch as tc
from .net import feature_net, classifier
from PIL import Image
from torchvision import models, transforms
from pathlib import Path
from urllib import request
import os
import sys
def get_feature(jpgs,model_label='vgg',batch_size=4):
    
    featurenet = feature_net(model_label)
    img_transform = transforms.Compose([
    transforms.Resize(320),
    transforms.CenterCrop(299),
    transforms.ToTensor(),
    transforms.Normalize((0.5, 0.5, 0.5), (0.5, 0.5, 0.5))])
    features = []
    groups = len(jpgs) // batch_size
    def get_img(img_name):
        img = Image.open(img_name)
        if img.mode=='RGBA':
            img.mode = 'RGB'
        return img_transform(img).unsqueeze(0)
    if len(jpgs) % batch_size!=0:
        groups += 1
    for i in range(groups):
        t = [get_img(name) for name in jpgs[i*batch_size:(i+1)*batch_size]]
        f = tc.cat(t,0)
        features.append(featurenet(f))
    
    return tc.cat(features,0)
def get_dog_or_cat(feature):
    mynet = classifier(512, 2)
    if sys.platform.startswith('win'):
        p = Path(os.environ['HOMEPATH']) / '.yxspkg' / 'dog_cat'
        p = os.environ['HOMEDRIVE'] / p
    else:
        p = Path(os.environ['HOME']) / '.yxspkg' / 'dog_cat'
    if not p.exists():
        os.makedirs(p)
    pth = p / 'feature_model.pth'
    if not pth.is_file():
        print('下载模型文件','https://github.com/blacksong/fragment/raw/master/dog_cat/feature_model.pth')
        request.urlretrieve('https://github.com/blacksong/fragment/raw/master/dog_cat/feature_model.pth',pth)
        print('模型文件下载完毕')
    mynet.load_state_dict(tc.load(pth))
    out = mynet(feature)
    _, pred = tc.max(out,1)
    return pred
def run(jpgs):
    m = get_feature(jpgs)
    t = get_dog_or_cat(m)
    return t
def main(jpgs=None):
    if jpgs is None:
        jpgs = sys.argv[1:]
    t = run(jpgs)
    d = ['cat','dog']
    for jpg,i in zip(jpgs,t):
        print(jpg,d[i.item()])
if __name__=='__main__':
    main()
    



# 这是一个matplotlib显示中文的库，只要你在文件开头调用即可
# 调用方式
# from yxspkg import matplotlib_chinese

from pathlib import Path 
import sys
import os
import json
from urllib import request
from matplotlib import font_manager
import matplotlib as mpl

if sys.platform.startswith('linux'):
    p = Path(os.environ['HOME']) /'.cache'/'matplotlib'
    json_path = list(p.glob('*.json'))[0]
    json_data = json.load(open(json_path))
    need_install_simhei = True
    for i in json_data['ttflist']:
        if i['name'] == 'SimHei':
            need_install_simhei = False
            break
    else:
        
        need_install_simhei = True
        url = 'https://github.com/blacksong/fragment/raw/master/simhei.ttf'
        yxspkg_path = Path(os.environ['HOME']) /'.yxspkg' /'matplotlib'
        if not yxspkg_path.exists():
            os.makedirs(yxspkg_path)
        ttf = yxspkg_path/'SimHei.ttf'
        if not ttf.exists():
            print('Downloading SimHei.ttf from GitHub:{}'.format(url))
            request.urlretrieve(url,ttf)
        myfont = font_manager.FontProperties(fname=str(ttf)) 
        ttf_info = {
            'fname':str(ttf),
            'name':myfont.get_name(),
            'style':myfont.get_style(),
            'variant':myfont.get_variant(),
            'weight':myfont.get_weight(),
            'stretch':myfont.get_stretch(),
            'size':myfont.get_size(),
            '__class__':'FontEntry'
            }
        json_data['ttflist'].append(ttf_info)
        json.dump(json_data,open(json_path,'w'),indent=4, separators=(',', ': '))
    mpl.rcParams['font.sans-serif'] = ['SimHei'] 
    mpl.rcParams['axes.unicode_minus'] = False



from setuptools import setup  ,find_packages
import yxspkg
import sys


setup(name='yxspkg',   
      version=yxspkg.__version__,    
      description='My pypi module',    
      author='Blacksong',    
      author_email='blacksong@yeah.net',       
      url='https://github.com/blacksong',
      packages=find_packages(), 
      entry_points={
        'console_scripts':['dog_cat=yxspkg.dog_cat.dog_cat:main',
        'fafa_excel=yxspkg.fafa_excel:main',
        'songzgif=yxspkg.songzgif.gif:main',
        'yxspkg_pip=yxspkg.pip:main']
      },
      classifiers = ["Programming Language :: Python :: 3","Operating System :: OS Independent","Topic :: Software Development :: Libraries :: Python Modules"],
)   




from PyQt5 import QtCore, QtGui
from PyQt5.QtWidgets import QLabel,QWidget,QApplication,QPushButton,QCheckBox,QHBoxLayout,QVBoxLayout
from PyQt5.QtGui import QImage,QPixmap,QCursor,QPainter,QColor
import sys
from PIL import Image
import numpy as np
def ndarry2qimage(npimg): #ndarry图片转化为qimage图片
    if npimg.dtype!='uint8':
        npimg=npimg.astype('uint8')
    shape=npimg.shape
    if len(shape)==3 and shape[2]==4:
        return QImage(npimg.tobytes(),shape[1],shape[0],shape[1]*shape[2],QImage.Format_RGBA8888)
    if len(shape)==2:
        npimg=stack((npimg,npimg,npimg),2)
        shape=npimg.shape
    s=QImage(npimg.tobytes(),shape[1],shape[0],shape[1]*shape[2],QImage.Format_RGB888)
    return s
class shadowLabel(QLabel):
    def __init__(self,panel,parent):
        super().__init__(panel)
        self.parent=parent
        self.setStyleSheet('QWidget{background-color:rgba(0,0,0,80)}')
class HCutLabel(QLabel):
    LineWidth=4
    LineLength=12
    def __init__(self,panel,parent):
        super().__init__(panel)
        self.parent=parent
        self.setStyleSheet('QWidget{background-color:rgb(0,0,0)}')
        self.resize(self.LineLength,self.LineWidth)
        self.pos_name=None
    def mousePressEvent(self,e):
        
        self.parent.ispressed=self.pos_name.split('_')[0]
        self.parent.wpos=QCursor.pos().x(),QCursor.pos().y()
    def mouseReleaseEvent(self,e):
        self.parent.ispressed=None
    def enterEvent(self,e):
        self.setStyleSheet('QWidget{background-color:rgb(90,90,90)}')
    def leaveEvent(self,e):
        self.setStyleSheet('QWidget{background-color:rgb(0,0,0)}')
class VCutLabel(HCutLabel):
    def __init__(self,panel,parent):
        super().__init__(panel,parent)
        self.resize(self.LineWidth,self.LineLength)
class radio_click_text_label(QLabel):
    def __init__(self,d):
        super().__init__(d)
        pass
class _ImageLabel(QLabel):
    def __init__(self,d,parent = None):
        super().__init__(d)
        self.parent = parent
        self.mask = None
        self.move_cut_widget = False
    def mouseMoveEvent(self,e):
        if self.mask is not None:
            h,w = self.img.shape[:2]
            w0,h0 = self.width(),self.height()
            rate = h0 / h
            x,y = int(e.x()//rate),int(e.y()//rate)
            for j in range(max(0,y-self.pen_width),min(self.img.shape[0], y+self.pen_width)):
                for i in range(max(0,x-self.pen_width),min(self.img.shape[1],x+self.pen_width)):
                    self.mask[j,i] = self.mask_type
            if self.mask_type == 0:
                mcolor = 0
            else:
                mcolor = (255,0,0)
            self.img[self.mask == self.mask_type] = mcolor
            q = QPixmap.fromImage(ndarry2qimage(self.img))
            self.setPixmap(q)
        elif self.move_cut_widget:
            x,y = QCursor.pos().x(),QCursor.pos().y()
            x0,y0 = self.mouse_pos0
            x1,y1,x2,y2 = self.cutpart0
            dx = x2-x1 
            dy = y2-y1
            x1 += x-x0
            y1 += y-y0
            self.parent.moveto(x1,y1,x1+dx,y1+dy,is_move=True)


    def mousePressEvent(self,e):
        self.move_cut_widget = True
        self.mouse_pos0=QCursor.pos().x(),QCursor.pos().y()
        self.cutpart0 = self.parent.cutpart
    def releasePressEvent(self,e):
        self.move_cut_widget = False
    def setGrab(self,m):
        if self.mask is None:
            p = self.pixmap()
            self.img = np.fromstring(Image.fromqpixmap(p).tobytes(),dtype = 'uint8').reshape((p.height(),p.width(),-1))
            self.img = self.img[:,:,:3]
            self.mask = self.img[:,:,0] * 0 + 3
            self.pen_width = self.img.shape[0] // 50
        self.mask_type = m

class CutImageWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.cutpart=(20,20,70,70)
        self.panel = QWidget(self)
        self.imageLabel=_ImageLabel(self.panel,parent = self)
        self.imageLabel.setScaledContents(True)
        self.geometry_image=self.cutpart
        # self.imageLabel.setSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored)
        self.create_cut_tool(*self.cutpart)
        # self.resize(500,500)
        self.setMinimumSize(524,600)
        self.panel.resize(500,540)
        self.ispressed=None
        self.MinSize=(10,10)
        self.wpos=None  #鼠标初始位置
        self.moveto(*self.cutpart)
        self.ok_button=QPushButton(self)
        self.ok_button.setText('OK')
        self.ok_button.clicked.connect(self.ok_event)
        self.ok_func=lambda x:None

        self.ratiobox=QCheckBox('应用于全部',self)

        self.ratiobox.stateChanged.connect(self.applyAll)
        self.ratiobox.toggle()  #设置默认值 self.apply_all=True

        self.keepbox=QCheckBox('1:1',self)
        self.keepbox.stateChanged.connect(self.checkRatio)
        self.keep_ratio=False
        self.cut_rate = None
        self.layout = QVBoxLayout()
        self.button_layout = QHBoxLayout()
        self.layout.addWidget(self.panel)
        self.button_layout.addWidget(self.keepbox)
        self.button_layout.addWidget(self.ratiobox)
        self.button_layout.addStretch()
        self.button_layout.addWidget(self.ok_button)
        self.layout.addLayout(self.button_layout)
        self.setLayout(self.layout)
    def checkRatio(self,state):
        if state == QtCore.Qt.Checked:
            self.keep_ratio=True
            x1,y1,x2,y2=self.cutpart
            t=min(x2-x1,y2-y1)
            self.moveto(x1,y1,x1+t,y1+t)
        else:
            self.keep_ratio=False
        
    def resizeEvent(self,e):
        w,h = self.width(),self.height()
    def applyAll(self,state):
        if state == QtCore.Qt.Checked:
            self.apply_all=True
        else:
            self.apply_all=False
    def ok_event(self,e):
        self.destroy()
        x1,y1,x2,y2=self.cutpart
        self.ok_func({'img_geometry':self.geometry_image,'cutpart_geometry':(x1,y1,x2-x1,y2-y1),'isall':self.apply_all})
    def ok_connect(self,func):
        self.ok_func=func
    def create_cut_tool(self,x1=10,y1=10,x2=70,y2=70):
        self.shadow_up=shadowLabel(self.panel,self)
        self.shadow_bottom=shadowLabel(self.panel,self)
        self.shadow_right=shadowLabel(self.panel,self)
        self.shadow_left=shadowLabel(self.panel,self)

        self.up_left_label=HCutLabel(self.panel,self)
        self.up_left_label.pos_name='up_left'
        self.up_middle_label=HCutLabel(self.panel,self)
        self.up_middle_label.pos_name='up_middle'
        self.up_right_label=HCutLabel(self.panel,self)
        self.up_right_label.pos_name='up_right'

        self.bottom_left_label=HCutLabel(self.panel,self)
        self.bottom_left_label.pos_name='bottom_left'
        self.bottom_middle_label=HCutLabel(self.panel,self)
        self.bottom_middle_label.pos_name='bottom_middle'
        self.bottom_right_label=HCutLabel(self.panel,self)
        self.bottom_right_label.pos_name='bottom_right'

        self.right_up_label=VCutLabel(self.panel,self)
        self.right_middle_label=VCutLabel(self.panel,self)
        self.right_bottom_label=VCutLabel(self.panel,self)
        self.right_up_label.pos_name='right_up'
        self.right_middle_label.pos_name='right_middle'
        self.right_bottom_label.pos_name='right_bottom'

        self.left_up_label=VCutLabel(self.panel,self)
        self.left_middle_label=VCutLabel(self.panel,self)
        self.left_bottom_label=VCutLabel(self.panel,self)
        self.left_up_label.pos_name='left_up'
        self.left_middle_label.pos_name='left_middle'
        self.left_bottom_label.pos_name='left_bottom'

    def moveto(self,x1=10,y1=10,x2=70,y2=70,is_move=False):
        width=HCutLabel.LineWidth
        length=HCutLabel.LineLength
        window_length,window_width=self.panel.height(),self.panel.width()
        mx1,my1,mx2,my2=self.geometry_image
        mx2+=mx1
        my2+=my1

        def checkCutPart(x1,y1,x2,y2):
            if is_move:
                dx = x2 - x1 
                dy = y2 - y1
                if x1<mx1:
                    x1,x2 = mx1,mx1+dx
                if x2>mx2:
                    x1,x2 = mx2-dx,mx2 
                if y1<my1:
                    y1,y2 = my1,my1+dy 
                if y2>my2:
                    y1,y2 = my2-dy,my2 
                return x1,y1,x2,y2
            else:
                if x2-x1>self.MinSize[0] and y2-y1>self.MinSize[1] and x1>=mx1 and y1>=my1 and x2<=mx2 and y2<=my2:
                    return x1,y1,x2,y2 
                else:
                    return False
        t=checkCutPart(x1,y1,x2,y2)
        if t:
            self.cutpart=t
            x1,y1,x2,y2 = t
        else:
            return False
        self.up_left_label.move(x1-width,y1-width)
        self.up_middle_label.move((x2+x1-length)/2,y1-width)
        self.up_right_label.move(x2+width-length,y1-width)

        self.bottom_left_label.move(x1-width,y2)
        self.bottom_middle_label.move((x2+x1-length)/2,y2)
        self.bottom_right_label.move(x2+width-length,y2)

        self.right_up_label.move(x2,y1-width)
        self.right_middle_label.move(x2,(y2+y1-length)/2)
        self.right_bottom_label.move(x2,y2+width-length)

        self.left_up_label.move(x1-width,y1-width)
        self.left_middle_label.move(x1-width,(y2+y1-length)/2)
        self.left_bottom_label.move(x1-width,y2+width-length)

        self.shadow_up.setGeometry(0,0,window_width,y1)
        self.shadow_bottom.setGeometry(0,y2,window_width,window_length-y2)
        self.shadow_left.setGeometry(0,y1,x1,y2-y1)
        self.shadow_right.setGeometry(x2,y1,window_width-x2,y2-y1)
        self.sizeChangeEvent({'img_geometry':self.geometry_image,'cutpart_geometry':(x1,y1,x2-x1,y2-y1)})
        return True
    def mouseMoveEvent(self,e):

        if not self.ispressed:return
        wpos=self.wpos
        mpos=QCursor.pos().x(),QCursor.pos().y()
        x,y=mpos[0]-wpos[0],mpos[1]-wpos[1]
        x1,y1,x2,y2=self.cutpart
        if self.ispressed == 'left':
            x1+=x
            if self.keep_ratio:y2-=x
        elif self.ispressed == 'right':
            x2+=x
            if self.keep_ratio:y2+=x
        elif self.ispressed =='up':
            y1+=y
            if self.keep_ratio:x2-=y
        elif self.ispressed == 'bottom':
            y2+=y
            if self.keep_ratio:x2+=y
        
        t=self.moveto(x1,y1,x2,y2)
        if t:
            self.wpos=mpos
    def setImage(self,qimage,geometry=None,ratio=0.8):
        self.qimage = qimage
        self.imageLabel.setPixmap(QPixmap.fromImage(qimage))
        height_window,width_window=self.panel.height(),self.panel.width()
        height_image,width_image=qimage.height(),qimage.width()
        if geometry is None:
            height=height_window*ratio
            width=height/height_image*width_image
            if width>ratio*width_window:
                width=width_window*ratio
                height=width/width_image*height_image
            x=(width_window-width)/2
            y=(height_window-height)/2
            geometry=(int(x),int(y),int(width),int(height))
        self.geometry_image=geometry
        self.imageLabel.setGeometry(*geometry)
        self.moveto(geometry[0],geometry[1],geometry[0]+geometry[2],geometry[1]+geometry[3])

    def sizeChangeEvent(self,e):
        pass
class GrabImageWidget(CutImageWidget):
    def __init__(self,*d):
        super().__init__(*d)
        self.back_button = QPushButton('标记背景',self)
        self.front_button = QPushButton('标记前景',self)
        self.back_button.clicked.connect(self.set_back)
        self.front_button.clicked.connect(self.set_front)

        self.button_layout.insertWidget(0,self.back_button)
        self.button_layout.insertWidget(0,self.front_button)
        self.qp = QPainter()
        self.need_painted = False

        labels = [self.up_left_label,
        self.up_middle_label,
        self.up_right_label,

        self.bottom_left_label,
        self.bottom_middle_label,
        self.bottom_right_label,

        self.right_up_label,
        self.right_middle_label,
        self.right_bottom_label,

        self.left_up_label,
        self.left_middle_label,
        self.left_bottom_label]
        [i.hide() for i in labels]


    def set_back(self):
        self.imageLabel.setGrab(0)
    def set_front(self):
        self.imageLabel.setGrab(1)
    def ok_event(self,e):
        self.destroy()
        x1,y1,x2,y2=self.cutpart
        self.ok_func({'img_geometry':self.geometry_image,'cutpart_geometry':(x1,y1,x2-x1,y2-y1),'isall':self.apply_all,'mask':self.imageLabel.mask})

    
if __name__=='__main__':
    app=QApplication(sys.argv)
    tw=GrabImageWidget()
    tw.setImage(QImage('4.jpg'))
    tw.show()
    app.exec_()
    print(dir(QPixmap))